from PASTEC.utils.AbstractClassifierAdaptatorAgent import AbstractClassifierAdaptatorAgent
import re

class AbstractClassifierAgent(AbstractClassifierAdaptatorAgent):
    
    CI_MAX = 1
    INIT_CLASSIF = "!"
    PATTERN_2_SEARCH_IN_BLAST_RESULTS = ""
    CATEGORY_LENGTH_BOUND = []
    CONDITIONS = {}

    ## Constructor
    #
    # @param env Environment instance
    # @param projectName string project name
    #
    def __init__(self, env):
        AbstractClassifierAdaptatorAgent.__init__(self, env)
        self._listOfKnownAgents = ["Size_Agent", "TRSearch_Agent", "TE_BLRx_Agent", "TE_BLRtx_Agent", "TE_Profiles_Agent", "ORF_Agent"]
        self.reset()
    
    def reset(self):
        self._classification = self.INIT_CLASSIF
        self._CI = 0
        self._dPresenceData = {"classif_other": [],
                               "classif_BM_tblastx": [],
                               "classif_BM_blastx": [],
                               "classif_nb": 0,
                               "weighting": []}
#        self._lNumericData = []
        self._dNumericData = {
                              "sizeFull":[],
                              "sizeWithoutTR":[],
                              "coverage":[],
                              "coverageOnSubject":[],
                              "identity":[],
                              }
        
        
        self._conclusionsSearch = []
        self._conclusionsMatch = []
        
    def getPresenceDataDict(self):
        return self._dPresenceData
    
    def getNumericDataDict(self):
        return self._dNumericData
    
#    def getNumericDataList(self):
#        return self._lNumericData
        
    def getCI(self):
        return self._CI
    
    def setPresenceDataDict(self, dPresenceData):
        self._dPresenceData = dPresenceData
    
    def setNumericDataDict(self, dNumericData):
        self._dNumericData = dNumericData
        
#    def setNumericDataList(self, lNumericData):
#        self._lNumericData = lNumericData
    
    ## Give the classification and the confidence index from all analysis conclusions
    #
    # @return classification, normalized confidence index
    #
    def decide(self):
        isAConditionValid = False
        lOtherConditions = self.CONDITIONS["classif_other"]
        for iCondition in lOtherConditions:
            if iCondition.validate(self._dNumericData["sizeWithoutTR"], self._dPresenceData["classif_other"]):
                isAConditionValid = True
                self._CI += iCondition.getWeight()
                
        lBMConditions = self.CONDITIONS["classif_BM"]
        for iCondition in lBMConditions:
            if iCondition.validate(None, self._dPresenceData["classif_BM_tblastx"]):
                isAConditionValid = True
                self._CI += iCondition.getWeight()
            if iCondition.validate(None, self._dPresenceData["classif_BM_blastx"]):
                isAConditionValid = True
                self._CI += iCondition.getWeight()
                
        if self._dPresenceData["classif_nb"]:
            isAConditionValid = True
            self._CI += self._dPresenceData["classif_nb"]
                
        if isAConditionValid:
            self._weightCI()
        
        normalizedIC = self._normalizeIC()
        return self._classification, normalizedIC
    
    ## Create the appropriate message of classification, confidence index,consensus name and the dict of evidence
    #
    # @param classification string 
    # @param IC integer confidence index
    #
    # @return dict of message
    #
    def generateMessage(self, classification, IC):
        if classification == self.INIT_CLASSIF:
            IC = self._environment.getMaxCI()
        dictOfEvidence = self._fillDictOfEvidence(self.PATTERN_2_SEARCH_IN_BLAST_RESULTS)
        message = {'consensusName': self._consensusName, 'classif': classification, 'confidenceIndex': IC, 'dictOfEvidence': dictOfEvidence}
        return message
                
    def makeAppropriateConclusionsForSize_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None and iDemandAndResult.result is not None:
            iSize = iDemandAndResult.result
            consensusLengthWithoutTR = iSize.getSizeWithoutTR()
            lBound = sorted(self.CATEGORY_LENGTH_BOUND)
            msg = "<%sbps" % lBound[0]
            for bound in lBound :
                if consensusLengthWithoutTR > bound:
                    msg = ">%sbps" % bound
            self._conclusionsSearch.append(msg)
            self._dNumericData["sizeFull"].append(iSize.getSize())
            self._dNumericData["sizeWithoutTR"].append(iSize.getSizeWithoutTR())
            #self._lNumericData.append(iSize.getSizeWithoutTR())
                            
    def makeAppropriateConclusionsForTE_BLRx_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None and iDemandAndResult.result != []:
            self._conclusionsMatch.append({'Repbase_bx': iDemandAndResult.result})
            lBM = iDemandAndResult.result
            self._dPresenceData["classif_BM_blastx"] = [iBM.getName() for iBM in lBM]

    def makeAppropriateConclusionsForTE_BLRtx_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None and iDemandAndResult.result != []:
            self._conclusionsMatch.append({'Repbase_tbx': iDemandAndResult.result})
            lBM = iDemandAndResult.result
            self._dPresenceData["classif_BM_tblastx"] = [iBM.getName() for iBM in lBM]

    def makeAppropriateConclusionsForTE_Profiles_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None and iDemandAndResult.result:
            lPDM = iDemandAndResult.result
            self._dPresenceData["classif_other"].extend([iPDM.getType() for iPDM in lPDM])
            self._conclusionsMatch.append({'profiles': lPDM})

    def makeAppropriateConclusionsForORF_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None and iDemandAndResult.result:
            lORF = iDemandAndResult.result
            for iORF in lORF:
                self._dPresenceData["weighting"].extend(iORF.getProfileTypesList())
            self._conclusionsMatch.append({'ORF': lORF})

    def makeAppropriateConclusionsForTRSearch_Agent(self, iDemandAndResult):
        if iDemandAndResult is not None:
            lTR = iDemandAndResult.result
            self._dPresenceData["classif_other"].extend([iTR.getType() for iTR in lTR])
            self._makeAppropriateConclusionsForTRSearch_Agent(iDemandAndResult)

    def _weightCI(self):
        self._classification = self.getAgentName()
        lWeightingConditions = self.CONDITIONS["weighting"]
        for iCondition in lWeightingConditions:
            if iCondition.validate(self._dNumericData["sizeWithoutTR"], self._dPresenceData["weighting"]):
                self._CI += iCondition.getWeight()

    def _makeAppropriateConclusionsForTRSearch_Agent(self, iDemandAndResult):
        pass

    def _fillDictOfEvidence(self, motif2Search):#TODO : motif2Search should be PATTERN2Search
        dictOfEvidence = {}
        listOfTR = []
        listOfSSR = []
        dictOfHelitronsExtremities = {}
        dictOfProfiles = {}
        dictOfOTHERProfiles = {}
        for iConclusion in self._conclusionsSearch:
            strConclusion = str(iConclusion)
            if re.match(".*term.*", strConclusion):
                listOfTR.append(iConclusion)
            if re.match("polyAtail", strConclusion):
                dictOfEvidence['polyAtail'] = "yes"
            if re.search("_start|_end", strConclusion):
                listOfSSR.append(iConclusion)
            if re.match(".*bps", strConclusion):
                dictOfEvidence['length'] = iConclusion
        for conclusion in self._conclusionsMatch:
            if conclusion.has_key('Repbase_tbx'):
                lBM = []
                for iBlasterMatch in conclusion['Repbase_tbx']:
                    if re.search(motif2Search, iBlasterMatch.getName()):
                        lBM.append(iBlasterMatch)
                if lBM != []:
                    dictOfEvidence['Repbase_tbx'] = lBM
            if conclusion.has_key('Repbase_bx'):
                lBM = []
                for iBlasterMatch in conclusion['Repbase_bx']:
                    if re.search(motif2Search, iBlasterMatch.getName()):
                        lBM.append(iBlasterMatch)
                if lBM != []:
                    dictOfEvidence['Repbase_bx'] = lBM
            if conclusion.has_key('profiles'):
                for iPDM in conclusion['profiles']:
                    dictOfProfiles[iPDM.getName()] = iPDM
            if conclusion.has_key('profilesOTHER'):
                for iPDM in conclusion['profilesOTHER']:
                    dictOfOTHERProfiles[iPDM.getName()] = iPDM
            if conclusion.has_key('ORF'):
                dictOfEvidence['ORF'] = []
                for iORF in conclusion['ORF']:
                    dictOfEvidence['ORF'].append(str(iORF))
            if conclusion.has_key('helitronExtremities'):
                dictOfHelitronsExtremities = conclusion['helitronExtremities']
            if conclusion.has_key('HG'):
                dictOfEvidence["HG"] = conclusion["HG"]
            if conclusion.has_key('rDNA'):
                dictOfEvidence["rDNA"] = conclusion["rDNA"]
            if conclusion.has_key('SSR'):
                dictOfEvidence["SSRCoverage"] = conclusion["SSR"]
        if listOfTR != []:
            dictOfEvidence['TR'] = listOfTR
        if listOfSSR != []:
            dictOfEvidence['SSR'] = listOfSSR
        if dictOfProfiles != {}:
            dictOfEvidence['te_hmmer'] = dictOfProfiles
        if dictOfOTHERProfiles != {}:
            dictOfEvidence['Other_profiles'] = dictOfOTHERProfiles
        if dictOfHelitronsExtremities != {}:
            dictOfEvidence['helitronExtremities'] = dictOfHelitronsExtremities
        return dictOfEvidence
