import sys, os, shutil, tarfile, glob, subprocess, pickle, shelve
from commons.core.launcher.Launcher import Launcher
from commons.core.sql.TableJobAdaptatorFactory import TableJobAdaptatorFactory
from commons.core.utils.FileUtils import FileUtils

repetPath = os.environ["REPET_PATH"].rstrip(":") #bugfix because galaxy adds ":" at the end of the path

def _map(iLauncher, cmd, cmdStart, cmdFinish ):
	lCmds = []
	lCmds.extend(cmd)
	lCmdStart = []
	lCmdStart.extend(cmdStart)
	lCmdFinish = []
	lCmdFinish.extend(cmdFinish)
	return(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))

def splitFastQ(fileName, nbOfSeqPerBatch):
	nbOfLinesPerFile = nbOfSeqPerBatch * 4
	resDir = os.path.dirname(fileName)
	filePrefix, fileExt = os.path.splitext(os.path.basename(fileName))
	cmd = "split -l %s %s %s_cut. -d" % (nbOfLinesPerFile, fileName, filePrefix)
	os.system(cmd)
	lOutput = glob.glob("%s/%s_cut.*" % (resDir, filePrefix))
	return lOutput

def _createSplitFastQCommand(iLauncher, fileName, nbOfSeqPerBatch, pickleFile):
	cmd2Launch = []
	cmd2Launch.append("log = 0")
	cmd2Launch.append("import pickle, glob")
	cmd2Launch.append("fileName = '%s'" % fileName)
	cmd2Launch.append("pickleFile='%s'" % pickleFile)
	cmd2Launch.append("nbOfLinesPerFile = %s" % (nbOfSeqPerBatch * 4))
	cmd2Launch.append("resDir = os.path.dirname(fileName)")
	cmd2Launch.append("filePrefix, fileExt = os.path.splitext(os.path.basename(fileName))")
	cmd2Launch.append("cmd = 'split -l %s %s %s/%s_cut. -d' % (nbOfLinesPerFile, fileName, resDir, filePrefix)")
	cmd2Launch.append("os.system(cmd)")
	cmd2Launch.append("lOutput = glob.glob('%s/%s_cut.*' % (resDir, filePrefix))")
	cmd2Launch.append("pickle.dump(lOutput, file('%s/%s' % (resDir, pickleFile), 'w'))")
	return cmd2Launch

def _createJoinFastQCommand(iLauncher):
	cmd2Launch = []
	cmd2Launch.append("log = 0")
	cmd2Launch.append("import pickle")
	cmd2Launch.append("from commons.core.utils.FileUtils import FileUtils")
	cmd2Launch.append("dCutOut2Out = pickle.load(file('%s'))" % os.path.abspath("fastQ2join.pickle"))
	cmd2Launch.append("for key in dCutOut2Out.keys():")
	cmd2Launch.append("\tFileUtils.catFilesFromList(dCutOut2Out[key],key, False)")
	return cmd2Launch	
	
#def joinFastQ(dCutOut2Out):
#	for key in dCutOut2Out.keys():
#		FileUtils.catFilesFromList(dCutOut2Out[key],key, False)
		
def _createFastqGroomerCode(outGroomerNames, inputFileNames, input_type, output_type, force_quality_encoding, summarize_input):
	cmd2Launch = []
	cmd2Launch.append("log = 0")
	cmd2Launch.append("from galaxy_utils.sequence.fastq import fastqReader, fastqVerboseErrorReader, fastqAggregator, fastqWriter")
	cmd2Launch.append("aggregator = fastqAggregator()")
	cmd2Launch.append("out = fastqWriter( open( '%s', 'wb' ), format = '%s', force_quality_encoding = '%s')" % (outGroomerNames,output_type,force_quality_encoding))
	cmd2Launch.append("read_count = None")
	if summarize_input:
		cmd2Launch.append("reader = fastqVerboseErrorReader")
	else:
		cmd2Launch.append("reader = fastqReader")
	cmd2Launch.append("for read_count, fastq_read in enumerate( reader( open( '%s' ), format = '%s', apply_galaxy_conventions = True ) ):" % (inputFileNames, input_type))
	if summarize_input:
		cmd2Launch.append("\taggregator.consume_read( fastq_read )")
		cmd2Launch.append("\tout.write( fastq_read )")
	cmd2Launch.append("out.close()")
	cmd2Launch.append("if read_count is not None:")
	#cmd2Launch.append("\tprint 'Groomed %s %s reads into %s reads.' % ( read_count + 1, %s, %s )" % ('%i', '%s', '%s', input_type,output_type))
	cmd2Launch.append("\tif '%s' != '%s' and 'solexa' in [ '%s', '%s' ]:" % (input_type, output_type, input_type, output_type))
	cmd2Launch.append("\t\tprint 'Converted between Solexa and PHRED scores.'")	
	if summarize_input:
		cmd2Launch.append("\tprint 'Based upon quality and sequence, the input data is valid for: %s' % ( ', '.join( aggregator.get_valid_formats() )  or 'None' )")
		cmd2Launch.append("\tascii_range = aggregator.get_ascii_range()")
		cmd2Launch.append("\tdecimal_range =  aggregator.get_decimal_range()")
		cmd2Launch.append("\tprint 'Input ASCII range: %s(%i) - %s(%i)' % ( repr( ascii_range[0] ), ord( ascii_range[0] ), repr( ascii_range[1] ), ord( ascii_range[1] ) )")
		cmd2Launch.append("\tprint 'Input decimal range: %i - %i' % ( decimal_range[0], decimal_range[1] ) ")
	cmd2Launch.append("else:")
	cmd2Launch.append("\tprint 'No valid FASTQ reads were provided.'")
	cmd2Launch.append("\tlog = 255")
	return cmd2Launch

def stop_err(msg):
	sys.stderr.write("%s\n" % msg)
	sys.exit()
	
	
def _createUntarCommand(iLauncher, inputFile):
	lArgs = []
	lArgs.append("-u %s" % inputFile)
	lArgs.append("-p fastq_groomer")
	lArgs.append("-e fastq")
	lArgs.append("-w %s" % os.getcwd())
	
	return iLauncher.getSystemCommand("python %s/SMART/DiffExpAnal/manageTarForDEA.py " % repetPath, lArgs)

def _createTarCommand(iLauncher, inputFile, outputFile):
	lArgs = []
	lArgs.append("-t %s" % inputFile)
	lArgs.append("-o %s" % outputFile)
	return iLauncher.getSystemCommand("python %s/SMART/DiffExpAnal/manageTarForDEA.py " % repetPath, lArgs)

def main():

	input_filename = sys.argv[1]  #a txt file
	input_type = sys.argv[2]
	output_filename = sys.argv[3] #a txt file
	output_type = sys.argv[4]
	force_quality_encoding = sys.argv[5]
	summarize_input = sys.argv[6] == 'summarize_input'

	if force_quality_encoding == 'None':
		force_quality_encoding = None
	

	acronym = "untar"
	jobdb = TableJobAdaptatorFactory.createJobInstance()
	iLauncher = Launcher(jobdb, os.getcwd(), os.getcwd(), "", acronym, acronym, False, True)
	lCmdsTuples = []
	cmd2Launch = [_createUntarCommand(iLauncher,input_filename)]
	cmdStart = []
	cmdFinish = []
	lCmdsTuples.append(_map(iLauncher, cmd2Launch, cmdStart, cmdFinish))
	iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, True)
	
	inputFileNames = pickle.load(file(os.getcwd()+"/inputFile.pickle"))
	outGroomerNames = pickle.load(file(os.getcwd()+"/outputFile.pickle"))

			
	lCmdsTuples = []
	lCutInputFile = []
	lPickleFile = []
	for i in range(len(outGroomerNames)):
		pickleFile = "l2Cut.pickle%s" % i
		lPickleFile.append(pickleFile)
		acronym = "cutFastQ"
		jobdb2Cut = TableJobAdaptatorFactory.createJobInstance()
		iLauncher2Cut = Launcher(jobdb2Cut, os.getcwd(), os.getcwd(), "", acronym, acronym, False, True)
		cmd2Launch = _createSplitFastQCommand(iLauncher, inputFileNames[i], 800000, pickleFile)
		cmdStart = []
		cmdFinish = []
		lCmdsTuples.append(_map(iLauncher2Cut, cmd2Launch, cmdStart, cmdFinish))
	iLauncher2Cut.runLauncherForMultipleJobs(acronym, lCmdsTuples, True)

	for pickleFile in lPickleFile:
		lCutInputFile.append(pickle.load(file(pickleFile)))
		
	acronym = "fastqGroomer"
	jobdb = TableJobAdaptatorFactory.createJobInstance()
	iLauncher = Launcher(jobdb, os.getcwd(), os.getcwd(), "", acronym, acronym, False, True)
	lCmdsTuples = []	
	dCutOut2Out = {}
	for i in range(len(outGroomerNames)):
		lCutOutput = []
		for cutInput in lCutInputFile[i]:
			cutOutput = "%s_out" % cutInput
			lCutOutput.append(cutOutput)
			cmd2Launch = _createFastqGroomerCode(cutOutput, cutInput, input_type, output_type, force_quality_encoding, summarize_input)
			cmdStart = []
			cmdFinish = []		
			lCmdsTuples.append(_map(iLauncher, cmd2Launch, cmdStart, cmdFinish))
		dCutOut2Out[outGroomerNames[i]] = lCutOutput	
	pickle.dump(dCutOut2Out,file("fastQ2join.pickle", "w"))
	iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, False)
	
#	joinFastQ(dCutOut2Out)

	
	acronym = "joinFastQ"
	jobdb = TableJobAdaptatorFactory.createJobInstance()
	iLauncher = Launcher(jobdb, os.getcwd(), os.getcwd(), "", acronym, acronym, False, True)
	lCmdsTuples = []
	cmd2Launch =_createJoinFastQCommand(iLauncher)
	cmdStart = []
	cmdFinish = []
	lCmdsTuples.append(_map(iLauncher, cmd2Launch, cmdStart, cmdFinish))
	iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, True)
		
	files2TarName = "toTar.pickle"
	pickle.dump(outGroomerNames, file(files2TarName , "w"))
	acronym = "toTar"
	jobdb = TableJobAdaptatorFactory.createJobInstance()
	iLauncher = Launcher(jobdb, os.getcwd(), os.getcwd(), "", acronym, acronym, False, True)
	lCmdsTuples = []
	cmd2Launch =[_createTarCommand(iLauncher, os.path.abspath(files2TarName), output_filename)]
	cmdStart = []
	cmdFinish = []
	lCmdsTuples.append(_map(iLauncher, cmd2Launch, cmdStart, cmdFinish))
	iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, True)


if __name__ == "__main__": main()

