#! /usr/bin/env python2

import os
startupfilename = os.environ.get('PYTHONSTARTUP')
if startupfilename and os.path.isfile(startupfilename):
    execfile(startupfilename)

import sys
import re

import getopt
import exceptions
import logging

from pyRepet.seq.AlignedBioseqDB import *
#-----------------------------------------------------------------------------
def help():
        print "usage:",sys.argv[0]," [ options ]"
        print "option:"
        print "    -h: this help"
        print "    -t: tree file"
        print "    -f: alignment in fasta format"

def extract_term_branch(treefile):
    f=open(treefile)
    l=f.readlines()
    tree="".join(l).replace("\n","")
    prog=re.compile("\([^\)\(]+\:[^\)\(]+\,[^\)\(]+\:[^\)\(]+\)")
    res=prog.findall(tree)
    termbr=[]
    for i in res:
        s=i[1:-1]
        tok=s.split(",")
        if len(tok)==2:
            a,b=tok
            an,ad=a.split(":")
            bn,bd=b.split(":")
            termbr.append(((an,bn),(float(ad),float(bd))))
    return termbr

def extract_copy_len(seq):
    seq=seq.lstrip("-")
    seq=seq.rstrip("-")
    return len(seq)

def extract_size(seq1,seq2):
    for i in xrange(0,len(seq1.sequence)):
        if seq1.sequence[i]!='-' and seq2.sequence[i]!='-':
            start=i
            break
    for i in xrange(len(seq1.sequence)-1,-1,-1):
        if seq1.sequence[i]!='-' and seq2.sequence[i]!='-':
            end=i
            break
    count=0
    for i in xrange(start,end+1):
        if not (seq1.sequence[i]=='-' and seq2.sequence[i]=='-'):
            count+=1
    return count

def extract_internal_gaps(seq1,seq2):
    start1=-1
    lengap1=0
    lseg1=[]
    start2=-1
    lengap2=0
    lseg2=[]
    for i in xrange(0,len(seq1.sequence)):
        #print i,start1,lengap1,start2,lengap2
        if seq1.sequence[i]=='-':
            if start1==-1:
                start1=i
            if seq2.sequence[i]!='-':
                lengap1+=1
        else:
            if start1!=0 and lengap1!=0:
                lseg1.append((start1+1,lengap1))
                #print lseg1
            start1=-1
            lengap1=0

        if seq2.sequence[i]=='-':
            if start2==-1:
                start2=i
            if seq1.sequence[i]!='-':
                lengap2+=1
        else:
            if start2!=0 and lengap2!=0:
                lseg2.append((start2+1,lengap2))
                #print lseg2
            start2=-1
            lengap2=0

    return lseg1,lseg2

def extract_internal_gaps_with_termgap(seq1,seq2):
    start1=-1
    lengap1=0
    lseg1=[]
    start2=-1
    lengap2=0
    lseg2=[]
    for i in xrange(0,len(seq1.sequence)):
        #print i,start1,lengap1,start2,lengap2
        if seq1.sequence[i]=='-':
            if start1==-1:
                start1=i
            if seq2.sequence[i]!='-':
                lengap1+=1
        else:
            if lengap1!=0:
                lseg1.append((start1+1,lengap1))
                #print lseg1
            start1=-1
            lengap1=0

        if seq2.sequence[i]=='-':
            if start2==-1:
                start2=i
            if seq1.sequence[i]!='-':
                lengap2+=1
        else:
            if lengap2!=0:
                lseg2.append((start2+1,lengap2))
                #print lseg2
            start2=-1
            lengap2=0

    if lengap1!=0:
        lseg1.append((start1+1,lengap1))
    if lengap2!=0:
        lseg2.append((start2+1,lengap2))

    return lseg1,lseg2

def main():
    treefile=""
    alignfile=""
    
    opts, args = getopt.getopt(sys.argv[1:],"ht:f:")
    for o,a in opts:
        if o == "-h":
            help()
            return
        if o == "-t":
            treefile=a
        if o == "-f":
            alignfile=a

    if treefile=="" or alignfile=="":
        print "tree file or alignment file not given"
        help()
        sys.exit()
        
    term_branch=extract_term_branch(treefile)


    alignDB=AlignedBioseqDB()
    alignDB.load(alignfile)

    for i in term_branch:
        #print i
        seq1=alignDB.fetch(i[0][0])
        seq2=alignDB.fetch(i[0][1])
        gap1,gap2=extract_internal_gaps(seq1,seq2)
        size=extract_size(seq1,seq2)
        len_seq1=extract_copy_len(seq1.sequence)
        len_seq2=extract_copy_len(seq2.sequence)
        true_len_seq1=alignDB.true_length(seq1.sequence)
        true_len_seq2=alignDB.true_length(seq2.sequence)
        if len(gap1)==0:
            print "%s\t%f\t%d\t%d\t%d\t%d"%(seq1.header,i[1][0],0,size,true_len_seq1,seq1.length())
        else:
            for j in gap1:
                #print j
                if i[1][0]>=0:
                    print "%s\t%f\t%d\t%d\t%d\t%d"%(seq1.header,i[1][0],j[1],size,true_len_seq1,seq1.length())
        if len(gap2)==0:
            print "%s\t%f\t%d\t%d\t%d\t%d"%(seq2.header,i[1][1],0,size,true_len_seq2,seq2.length())
        else:
            for j in gap2:
                #print j
                if i[1][1]>=0:
                    print "%s\t%f\t%d\t%d\t%d\t%d"%(seq2.header,i[1][1],j[1],size,true_len_seq2,seq2.length())

if __name__ == "__main__": 
    main ()

    
