#!/usr/bin/env python

##@file launch all steps of the TEannot pipeline
# usage: launch_TEannotByStep.py [ options ]
# options:
#      -h: this help
#      -P: project name
#      -C: configuration file (default=TEdenovo.cfg)
#      -S: step(s) (default='12345678', '1', '23', ...)"
#      -v: verbose (default=0/1/2/...)

import sys
import os
import getopt
import time


def help():
    print
    print "usage: %s [ options ]" % ( sys.argv[0] )
    print "options:"
    print "     -h: this help"
    print "     -P: project name"
    print "     -C: configuration file (default=TEannot.cfg)"
    print "     -S: step(s) (default='12345678', '1', '23', ...)"
    print "     -O: output format (GFF3 or gameXML) (default=GFF3)"
    print "     -v: verbose (default=0/1/2/...)"
    print
    
    
def main():
    projectName = ""
    configFileName = "TEannot.cfg"
    steps = "12345678"
    verbose = 0
    format = "GFF3"
    try:
        opts, args = getopt.getopt( sys.argv[1:], "hP:C:S:O:v:" )
    except getopt.GetoptError, err:
        print str(err); help(); sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help(); sys.exit(0)
        elif o == "-P":
            projectName = a
        elif o == "-C":
            configFileName = a
        elif o == "-S":
            steps = a
        elif o == "-O":
            format = a
        elif o == "-v":
            verbose = int(a)
            
    if projectName == "":
        print "ERROR: missing project name (-P)"
        help(); sys.exit(1)
        
    if configFileName == "":
        print "ERROR: missing configuration file (-C)"
        help(); sys.exit(1)
        
    if not os.path.exists( configFileName ):
        print "ERROR: file '%s' doesn't exist" % ( configFileName )
        help(); sys.exit(1)
        
    if steps == "":
        print "ERROR: missing step(s) (-S)"
        help(); sys.exit(1)
        
    print "START time: %s" % ( time.strftime("%Y-%m-%d %H:%M:%S") )
    sys.stdout.flush()
    
    lCmds = []
    if "1" in steps:
        lCmds.append( "TEannot.py -P %s -C %s -S 1 -v %i" % ( projectName, configFileName, verbose ) )
    if "2" in steps:
        lCmds.append( "TEannot.py -P %s -C %s -S 2 -a BLR -v %i" % ( projectName, configFileName, verbose ) )
        lCmds.append( "TEannot.py -P %s -C %s -S 2 -a RM -v %i" % ( projectName, configFileName, verbose ) )
        lCmds.append( "TEannot.py -P %s -C %s -S 2 -a CEN -v %i" % ( projectName, configFileName, verbose ) )
        lCmds.append( "TEannot.py -P %s -C %s -S 2 -a BLR -r -v %i" % ( projectName, configFileName, verbose ) )
        lCmds.append( "TEannot.py -P %s -C %s -S 2 -a RM -r -v %i" % ( projectName, configFileName, verbose ) )
        lCmds.append( "TEannot.py -P %s -C %s -S 2 -a CEN -r -v %i" % ( projectName, configFileName, verbose ) )
    if "3" in steps:
        lCmds.append( "TEannot.py -P %s -C %s -S 3 -c BLR+RM+CEN -v %i" % ( projectName, configFileName, verbose ) )
    if "4" in steps:
        lCmds.append( "TEannot.py -P %s -C %s -S 4 -s TRF -v %i" % ( projectName, configFileName, verbose ) )
        lCmds.append( "TEannot.py -P %s -C %s -S 4 -s Mreps -v %i" % ( projectName, configFileName, verbose ) )
        lCmds.append( "TEannot.py -P %s -C %s -S 4 -s RMSSR -v %i" % ( projectName, configFileName, verbose ) )
    if "5" in steps:
        lCmds.append( "TEannot.py -P %s -C %s -S 5 -v %i" % ( projectName, configFileName, verbose ) )
    if "6" in steps:
        lCmds.append( "TEannot.py -P %s -C %s -S 6 -b tblastx -v %i" % ( projectName, configFileName, verbose ) )
        lCmds.append( "TEannot.py -P %s -C %s -S 6 -b blastx -v %i" % ( projectName, configFileName, verbose ) )
    if "7" in steps:
        lCmds.append( "TEannot.py -P %s -C %s -S 7 -v %i" % ( projectName, configFileName, verbose ) )
    if "8" in steps:
        lCmds.append( "TEannot.py -P %s -C %s -S 8 -o %s -v %i" % ( projectName, configFileName, format, verbose ) )
        
    for cmd in lCmds:
        returnValue = os.system( cmd )
        if returnValue != 0:
            print "ERROR: command '%s' returned %i" % ( cmd, returnValue )
            sys.exit(1)
            
    print "END time: %s" % ( time.strftime("%Y-%m-%d %H:%M:%S") )
    sys.stdout.flush()
    
    return 0
    
if __name__== "__main__":
    main()
