#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from commons.core.sql.DbFactory import DbFactory
from commons.core.sql.TableSetAdaptator import TableSetAdaptator
from commons.core.sql.TablePathAdaptator import TablePathAdaptator
from commons.core.sql.TableMapAdaptator import TableMapAdaptator
from commons.core.coord.SetUtils import SetUtils
import os
import sys
import getopt

#-----------------------------------------------------------------------------

def help():
    """
    Give the list of the command-line options.
    """
    print ""
    print "usage:",sys.argv[0]," [ options ]"
    print "option:"
    print "    -h: this help"
    print "    -f: name of the file in which are recorded tables and commands."
    print "         Structure of the command file:"
    print "          one line per command, each field separated by one tabulation."
    print "         Fields in column order:"
    print "         <table_name>: Name of the table to insert"
    print "         <tableType>: Type of the table. Values=[path|set]"
    print "         <insertType>: insertion type.Values=[merge]"
#    print "         <insertType>: insertion type.Values=[merge|diff|add|sub]"
#    print "         <rename_type>=<new_name>: Change name before insertion."
#    print "           Rename type values=[prefix|replace|split]"
#TODO: see pyRepet.sql.TableManip.add_table to implement "insert_type" and "rename_type" options
    print "    -o: name of the output table ('set' format)"
    print "     -v: verbose (default=0/1/2)"
    print ""

#-----------------------------------------------------------------------------

#TODO: to test !
#=> RunFunctionalTests.py -n TEannot -S 5 -c
# In results:
# (AATGTTT)4      dmel_chr4       76023   76339
# (TAA)n#Simple_repeat    dmel_chr4       76279   76107
# Why keep the second set ?

def main():

    cmdFileName = ""
    outTable = ""
    verbose = 0

    try:
        opts = getopt.getopt(sys.argv[1:],"hf:o:v:")[0]
    except getopt.GetoptError:
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(1)
        elif o == "-f":
            cmdFileName = a
        elif o == "-o":
            outTable = a
        elif o == "-v":
            verbose = int(a)

    if cmdFileName == "" or outTable == "":
        print "*** Error: missing compulsory options"
        help()
        sys.exit(1)

    if verbose > 0:
        print "\nbeginning of %s" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    db = DbFactory.createInstance()
    db.dropTable(outTable)

    cmdFile = open(cmdFileName)
    lines = cmdFile.readlines()
    cmdFile.close()

    # for each input table
    for l in lines:

        tok = l[:-1].split("\t")
        if len(tok) == 3:
            inTable, tableType, insertType = tok  
        else:
            print "*** Error when parsing '%s', found %d tokens in line '%s'" % (cmdFileName, len(tok), l)
            sys.exit(1)
        if inTable[0] == "#":
            continue
        if verbose > 1:
            print "table %s (type:%s) added to %s --> operation: merge" % (inTable, tableType, outTable)
            sys.stdout.flush()
        if tableType == "set":
            iInTA = TableSetAdaptator(db, inTable)
        elif tableType == "path":
            iInTA = TablePathAdaptator(db, inTable)
        elif tableType == "map":
            iInTA = TableMapAdaptator(db, inTable)
        else:
            print "*** Error: table type %s not implemented" % tableType
            sys.exit(1)
        tmp_filename = "%s.tmp%s-%s" % (outTable, str(os.getpid()), inTable)
        iOutTSA = TableSetAdaptator(db, outTable)
    
        if not db.doesTableExist(outTable):
            lQueryNames = iInTA.getSeqNameList()
            for queryName in lQueryNames:
                
                if verbose > 0:
                    print "sequence %s from %s table" % (queryName, inTable)
                    sys.stdout.flush()
                    
                lSets = iInTA.getSetListFromSeqName(queryName)
                lMergedSets = SetUtils.mergeSetsInList(lSets)
                lMergedSetsWithoutDuplicated = SetUtils.getSetListWithoutDuplicates(lMergedSets)
                SetUtils.writeListInFile(lMergedSetsWithoutDuplicated, tmp_filename, "a")
    
            if verbose > 1:
                print "create destination table"
            db.createTable(outTable, "set", tmp_filename)
            os.remove(tmp_filename)
            
        else:
            lInQueryNames = iInTA.getSeqNameList()
            lOutQueryNames = iOutTSA.getSeqNameList()
            newid = iOutTSA.getNewId()
            for inQueryName in lInQueryNames:
                
                if verbose > 0:
                    print "sequence",inQueryName,"from",inTable,"table"
                    sys.stdout.flush()
                    
                lInSets = iInTA.getSetListFromSeqName(inQueryName)
                lOutSets = iOutTSA.getSetListFromSeqName(inQueryName)
                if len(lOutSets) > 0:
                    lOutQueryNames.remove(inQueryName)
                if insertType == "merge":
                    lInSets = SetUtils.mergeSetsInList(lInSets)
                    lMergedSets, newid = SetUtils.getListOfMergedSetsAndNextId(lOutSets, lInSets, newid)
#                elif insertType == "add":
#                    lMergedSets, newid = SetUtils.getSetsListOfTwoConcatenatedSetsListAndNextId(lOutSets, lInSets, newid)
#                elif insertType == "diff":
#                    lMergedSets, newid = pyRepet.coord.Set.set_list_add_diff_set( lOutSets, lInSets, newid )
#                elif insertType == "sub":
#                    lMergedSets, newid = pyRepet.coord.Set.set_list_sub_set( lOutSets, lInSets, newid )
                else:
                    print "*** Error: insert type",insertType,"not implemented"
                    sys.exit(1)
                lMergedSetsWithoutDuplicated = SetUtils.getSetListWithoutDuplicates(lMergedSets)
                SetUtils.writeListInFile(lMergedSetsWithoutDuplicated, tmp_filename, "a")
        
            if verbose > 0:
                print "write remaining dest contigs",lOutQueryNames
            for outQueryName in lOutQueryNames:
                
                if verbose > 1:
                    print "remain sequence",outQueryName
                    
                lOutSets = iOutTSA.getSetListFromSeqName(outQueryName)
                dId2SetsList = SetUtils.getDictOfListsWithIdAsKey(lOutSets)
                lMergedSets = []
                for lSets in dId2SetsList.values():
                    SetUtils.changeIdInList(lSets, newid)
                    lSetsWithoutDuplicated = SetUtils.getSetListWithoutDuplicates(lSets)
                    lMergedSets.extend(lSetsWithoutDuplicated)
                    newid += 1
                SetUtils.writeListInFile(lSetsWithoutDuplicated, tmp_filename, "a")
        
            if verbose > 0:
                print "create destination table"
            db.createTable(outTable, "set", tmp_filename, True)
            os.remove(tmp_filename)

    if verbose > 0:
        print "%s finished successfully\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    return 0

#----------------------------------------------------------------------------

if __name__ == '__main__':
    main()
