#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

class ClassifHierarchyContainer(object):
    
    def __init__(self, classifFileName = "", dClassif = {}, elementName = "Transposable Element"):
        self._classifFileName = classifFileName
        self._dClassif = dClassif
        self._TEName = elementName
        
    def __eq__(self, o):
        if type(o) is type(self):
            return self._classifFileName == o._classifFileName and self._dClassif == o._dClassif
        return False
    
    def __ne__(self, o):
        return not self.__eq__(o)
    
    def getClassifDict(self):
        return self._dClassif
    
    def getClassesList(self):
        return sorted(self._dClassif[self._TEName].keys())
    
    def getOrdersList(self):
        dClasses = self._dClassif[self._TEName]
        sOrders = set()
        for dClass in dClasses.values():
            for dSuperOrder in dClass.values():
                sOrders.update(set(dSuperOrder.keys()))
        sOrders.discard('?')
        return sorted(list(sOrders))
    
    def getSuperFamiliesList(self):
        dClasses = self._dClassif[self._TEName]
        sSuperFamilies = set()
        for dClass in dClasses.values():
            for dSuperOrder in dClass.values():
                for dOrder in dSuperOrder.values():
                    sSuperFamilies.update(set(dOrder.keys()))
        sSuperFamilies.discard('?')
        return sorted(list(sSuperFamilies))
    
    def getFamiliesList(self):
        dClasses = self._dClassif[self._TEName]
        sFamilies = set()
        for dClass in dClasses.values():
            for dSuperOrder in dClass.values():
                for dOrder in dSuperOrder.values():
                    for dSuperFamilies in dOrder.values():
                        sFamilies.update(set(dSuperFamilies.keys()))
        sFamilies.discard('?')
        return sorted(list(sFamilies))
    
    def getClassFromOrder(self, order):
        dClasses = self._dClassif[self._TEName]
        for className, dClass in dClasses.items():
            for dSuperOrder in dClass.values():
                if order in dSuperOrder.keys():
                    return className
        return ""
    
    def getClassAndOrderFromSuperFamily(self, superFamily, ignoreCase = False):
        dClasses = self._dClassif[self._TEName]
        for className, dClass in dClasses.items():
            for dSuperOrder in dClass.values():
                for orderName, dOrder in dSuperOrder.items():
                    if ignoreCase:
                        keys = [key.lower() for key in dOrder.keys()]
                        if superFamily.lower() in keys:
                            return className, orderName
                    else:
                        if superFamily in dOrder.keys():
                            return className, orderName
        return "", ""
    
    def getClassOrderAndSuperFamilyFromFamily(self, family, ignoreCase = False):
        dClasses = self._dClassif[self._TEName]
        for className, dClass in dClasses.items():
            for dSuperOrder in dClass.values():
                for orderName, dOrder in dSuperOrder.items():
                    for superFamilyName, dSuperFamily in dOrder.items():
                        if ignoreCase:
                            keys = [key.lower() for key in dSuperFamily.keys()]
                            if family.lower() in keys:
                                return className, orderName, superFamilyName
                        else:
                            if family in dSuperFamily.keys():
                                return className, orderName, superFamilyName
        return "", "", ""

    def parse(self):
        currentLevel1 = ""
        currentLevel2 = ""
        currentLevel3 = ""
        currentLevel4 = ""
        currentLevel5 = ""
        currentLevel6 = ""
        classifFileHandler = open(self._classifFileName, "r" )    
        while True:
            line = classifFileHandler.readline()
            if line == "":
                break
            tokens = line[:-1].split("\t")
            if len(tokens) == 1:
                currentLevel1 = tokens[0]
                self._dClassif[ currentLevel1 ] = {}
            elif len(tokens) == 2:
                currentLevel2 = tokens[1]
                self._dClassif[ currentLevel1 ][ currentLevel2 ] = {}
            elif len(tokens) == 3:
                currentLevel3 = tokens[2]
                self._dClassif[ currentLevel1 ][ currentLevel2 ][ currentLevel3 ] = {}
            elif len(tokens) == 4:
                currentLevel4 = tokens[3]
                self._dClassif[ currentLevel1 ][ currentLevel2 ][ currentLevel3 ] [ currentLevel4 ] = {}
            elif len(tokens) == 5:
                currentLevel5 = tokens[4]
                self._dClassif[ currentLevel1 ][ currentLevel2 ][ currentLevel3 ][ currentLevel4 ][ currentLevel5 ] = {}
            elif len(tokens) == 6:
                currentLevel6 = tokens[5]
                self._dClassif[ currentLevel1 ][ currentLevel2 ][ currentLevel3 ][ currentLevel4 ][ currentLevel5 ][ currentLevel6 ] = {}
            else:
                raise Exception("ERROR: wrong format for %s!" % self._classifFileName)
        classifFileHandler.close()
#        print "dico des classif"
#        print self._dClassif