#!/usr/bin/env python

import sys,os
import getopt
import string
from commons.core.coord.MapUtils import MapUtils
from commons.core.coord.Map import Map
from commons.core.seq.Bioseq import Bioseq

class MaskSeqFromCoord( object ):

    def __init__( self, inFileName = "", coordFileName = "", formatType = "map", softMasking = False, xMasking = False, outFileName = "", verbose = 0):
        self._inFileName = inFileName
        self._coordFileName = coordFileName
        self._formatType = formatType
        self._softMasking = softMasking
        self._xMasking = xMasking
        self._outFileName = outFileName
        self._verbose = verbose

    def help(self):
        print
        print "usage: %s [ options ]" % ( sys.argv[0].split("/")[-1] )
        print "options:"
        print "     -h: this help"
        print "     -i: name of the input file (format='fasta')"
        print "     -m: name of the file with the coordinates"
        print "     -f: format of the file with the coordinates (map, path (from REPET), pyramid (from piler), gff3)"
#TODO implementation gff3 format
#        print "     -f: format of the file with the coordinates (map, path (from REPET), pyramid (from piler), gff3)"
        print "     -s: soft masking (lower case instead of N's)"
        print "     -X: X masking (X instead of N's)"
        print "     -o: name of the output file (default=inFileName+'.mask')"
        print "     -v: verbose (default=0/1)"
        print
        
    def setAttributesFromCmdLine( self ):
        try:
            opts, args = getopt.getopt( sys.argv[1:], "hi:m:f:sXo:v:" )
        except getopt.GetoptError, err:
            sys.stderr.write( "%s\n" % str(err) )
            help()
            sys.exit(1)
        for o,a in opts:
            if o == "-h":
                help()
                sys.exit(0)
            elif o == "-i":
                self._inFileName = a
            elif o == "-m":
                self._coordFileName = a
            elif o == "-f":
                self._formatType = a
            elif o == "-s":
                self._softMasking = True
            elif o == "-X":
                self._xMasking =True
            elif o == "-o":
                self._outFileName = a
            elif o == "-v":
                self._verbose = int(a)

    def checkAttributes( self ):
        if self._inFileName == "":
            msg = "ERROR: missing input fasta file (-i)"
            sys.stderr.write( "%s\n" % msg )
            help()
            sys.exit(1)
        if self._formatType == "":
            msg = "ERROR: missing format (-f)"
            sys.stderr.write( "%s\n" % msg )
            help()
            sys.exit(1)
        if self._formatType not in [ "map", "pyramid", "path"]:
            msg = "ERROR: format '%s' not yet available" % ( self._formatType )
            sys.stderr.write( "%s\n" % msg )
            help()
            sys.exit(1)
    
        if self._outFileName == "":
            self._outFileName = "%s.mask" % ( self._inFileName )
            
        
    def getDictPerSeqNameFromPyramidFile( self, coordFileName ):
        """
        Load the coordinates to be masked.
        @param coordFileName: name of the 'pyramid' file
        @type coordFileName: string
        @return: dictionary whose keys are sequence names and values the coordinates
        """
        dSeqname2Maps = {}
        coordFile = open( coordFileName, "r" )
        while True:
            line = coordFile.readline()
            if line == "":
                break
            tokens = line.split("\t")
            tokens = ( tokens[2], tokens[0], tokens[3], tokens[4] )
            iMap = Map()
            iMap.setFromTuple( tokens )
            if not dSeqname2Maps.has_key( iMap.seqname ):
                dSeqname2Maps[ iMap.seqname ] = [ iMap ]
            else:
                dSeqname2Maps[ iMap.seqname ].append( iMap )
        coordFile.close()
        return dSeqname2Maps
    
    
    def maskSequences( self, inFileName, outFileName, dSeqname2Map, softMasking, xMasking ):
        """
        Mask the sequences in a fasta file by the coordinates recorded in a dictionary.
        @param inFileName: name of the input fasta file
        @type inFileName: string
        @param outFileName: name of the output fasta file
        @type outFileName: string
        @param dSeqname2Map: dictionary whose keys are sequence names and values the coordinates
        @type dSeqname2Map: dictionary
        @param softMasking: replace sequences to mask by lower case instead of N's
        @param softMasking: boolean
        @param xMasking: replace sequences to mask by X instead of N's
        @param xMasking: boolean
        """
        inFile = open( inFileName, "r" )
        outFile = open( outFileName, "w" )
        bs = Bioseq()
        while True:
            bs.reset()
            bs.read( inFile )
            if bs.header == None:
                break
            tmpSeq = list( bs.sequence )
            if dSeqname2Map.has_key( bs.header ):
                for iMap in dSeqname2Map[ bs.header ]:
#If xMasking and softMasking are True only xMasking will be considered
                    if xMasking : 
                        tmpSeq[ min(iMap.start,iMap.end)-1 : max(iMap.start,iMap.end) ] = (abs(iMap.end - iMap.start) + 1) * "X"
                    elif softMasking:
                        tmpSeq[ min(iMap.start,iMap.end)-1 : max(iMap.start,iMap.end) ] = map( string.lower, tmpSeq[ min(iMap.start,iMap.end)-1 : max(iMap.start,iMap.end) ] )
                    else:	
                        tmpSeq[ min(iMap.start,iMap.end)-1 : max(iMap.start,iMap.end) ] = (abs(iMap.end - iMap.start) + 1) * "N"
            bs.sequence = "".join( tmpSeq )
            bs.write( outFile )
        inFile.close()
        outFile.close()
    
    
    def run(self):
        """
        Use a 'map' file to replace the sequence between two coordinates by Ns.
        """
        self.checkAttributes()

        if self._verbose > 0:
            msg = "START %s" % ( sys.argv[0].split("/")[-1] )
            sys.stdout.write( "%s\n" % msg )
            sys.stdout.flush()
            
        if self._formatType == "map":
            dSeqname2Map = MapUtils.getDictPerSeqNameFromMapFile( self._coordFileName )
        elif self._formatType == "pyramid":
            dSeqname2Map = self.getDictPerSeqNameFromPyramidFile( self._coordFileName )
        elif self._formatType == "path":
            os.system('awk "{print \$5,\$2,\$3,\$4}" %s | sed "s/ /\t/g"> %s.tmp'%(self._coordFileName, self._coordFileName))
            dSeqname2Map = MapUtils.getDictPerSeqNameFromMapFile( self._coordFileName+".tmp" )
            os.system("rm {0}.tmp".format(self._coordFileName))
        if self._verbose > 0:
            msg = "nb of input sequences to be masked: %i" % ( len(dSeqname2Map.keys()) )
            sys.stdout.write( "%s\n" % msg )
            sys.stdout.flush()
            
        if self._verbose > 0:
            msg = "save the masked sequences in '%s'" % ( self._outFileName )
            sys.stdout.write( "%s\n" % msg )
            sys.stdout.flush()
        self.maskSequences( self._inFileName, self._outFileName, dSeqname2Map, self._softMasking, self._xMasking )
        
        if self._verbose > 0:
            msg = "END %s" % ( sys.argv[0].split("/")[-1] )
            sys.stdout.write( "%s\n" % msg )
            sys.stdout.flush()
            
        return 0

if __name__ == "__main__":
    iMSFC = MaskSeqFromCoord()
    iMSFC.setAttributesFromCmdLine()
    iMSFC.run()
