#!/usr/bin/env python

##@file launch all steps of the TEannot pipeline
# usage: launch_TEannot.py [ options ]
# options:
#      -h: this help
#      -P: project name
#      -C: configuration file (default=TEannot.cfg)
#      -t: multi-threaded
#      -e: export annotations (gameXML and GFF3)
#      -v: verbose (default=0/1/2/...)

import os
import sys
import getopt
import time
import ConfigParser
from threading import Thread

class MyThread( Thread ):
    def __init__( self, cmd ):
        Thread.__init__(self)
        self._cmd = cmd
    def run( self ):
        launchCmd( self._cmd )

def launchCmd( cmd ):
    log = os.system( cmd )
    if log != 0: print "ERROR"; sys.exit(1)

def launchUnthreaded():
    for i in xrange( 0, len(lCmds) ):
        launchCmd( lCmds[i] )

def launchThreaded():
    lThreads = []
    for i in xrange( 0, len(lCmds) ):
        time.sleep( 1 )
        lThreads.append( MyThread( lCmds[i] ) )
        lThreads[i].start()
    for i in xrange( 0, len(lCmds) ):
        lThreads[i].join()

def help():
    print
    print "usage: %s [ options ]" % ( sys.argv[0] )
    print "options:"
    print "     -h: this help"
    print "     -P: project name"
    print "     -C: configuration file (default=TEannot.cfg)"
    print "     -t: multi-threaded"
    print "     -e: export annotations (gameXML and GFF3)"
    print "     -v: verbose (default=0/1/2/...)"
    print

def main():

    global projectName
    projectName = ""
    configFileName = "TEannot.cfg"
    threaded = False
    exportAnnotations = False
    global verbose
    verbose = 0

    try:
        opts, args = getopt.getopt(sys.argv[1:],"hP:C:tev:")
    except getopt.GetoptError, err:
        print str(err); help(); sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help(); sys.exit(0)
        elif o == "-P":
            projectName = a
        elif o == "-C":
            configFileName = a
        elif o == "-t":
            threaded = True
        elif o == "-e":
            exportAnnotations = True
        elif o == "-v":
            verbose = int(a)

    if projectName == "":
        print "ERROR: missing project name"
        help()
        sys.exit(1)
        
    if configFileName == "":
        print "ERROR: missing configuration file"
        help()
        sys.exit(1)
        
    if not os.path.exists(configFileName):
        print "ERROR: file '%s' doesn't exist" % configFileName
        help(); sys.exit(1)

    print "START time: %s" % time.strftime("%Y-%m-%d %H:%M:%S")
    sys.stdout.flush()

    cmd = "TEannot.py -P %s -C %s -S 1 -v %i" % (projectName, configFileName, verbose)
    launchCmd( cmd )

    global lCmds
    lCmds = [
        "TEannot.py -P %s -C %s -S 2 -a BLR -v %i" % (projectName, configFileName, verbose),
        "TEannot.py -P %s -C %s -S 2 -a RM -v %i" % (projectName, configFileName, verbose),
        "TEannot.py -P %s -C %s -S 2 -a CEN -v %i" % (projectName, configFileName, verbose),
        "TEannot.py -P %s -C %s -S 2 -a BLR -r -v %i" % (projectName, configFileName, verbose),
        "TEannot.py -P %s -C %s -S 2 -a RM -r -v %i" % (projectName, configFileName, verbose),
        "TEannot.py -P %s -C %s -S 2 -a CEN -r -v %i" % (projectName, configFileName, verbose),
        "TEannot.py -P %s -C %s -S 4 -s TRF -v %i" % (projectName, configFileName, verbose),
        "TEannot.py -P %s -C %s -S 4 -s Mreps -v %i" % (projectName, configFileName, verbose),
        "TEannot.py -P %s -C %s -S 4 -s RMSSR -v %i" % (projectName, configFileName, verbose)
        ]
    config = ConfigParser.ConfigParser()
    config.readfp(open(configFileName))
    if os.path.exists(config.get("align_other_banks","bankBLRtx")):
        lCmds.append( "TEannot.py -P %s -C %s -S 6 -b tblastx -v %i" % (projectName, configFileName, verbose))
    if os.path.exists(config.get("align_other_banks","bankBLRx")):
        lCmds.append( "TEannot.py -P %s -C %s -S 6 -b blastx -v %i" % (projectName, configFileName, verbose))

    if threaded == False:
        launchUnthreaded()

    elif threaded == True:
        launchThreaded()

    cmd = "TEannot.py -P %s -C %s -S 3 -c BLR+RM+CEN -v %i" % (projectName, configFileName, verbose)
    launchCmd(cmd)
    cmd = "TEannot.py -P %s -C %s -S 5 -v %i" % (projectName, configFileName, verbose)
    launchCmd(cmd)
    cmd = "TEannot.py -P %s -C %s -S 7 -v %i" % (projectName, configFileName, verbose)
    launchCmd(cmd)
    
    if exportAnnotations:
        cmd = "TEannot.py -P %s -C %s -S 8 -o GFF3 -v %i" % (projectName, configFileName, verbose)
        launchCmd(cmd)
        cmd = "TEannot.py -P %s -C %s -S 8 -o gameXML -v %i" % (projectName, configFileName, verbose)
        launchCmd(cmd)

    print "END time: %s" % time.strftime("%Y-%m-%d %H:%M:%S")
    sys.stdout.flush()

    return 0

if __name__== "__main__":
    main()
