#!/usr/bin/env python

##@file launch all steps of the TEdenovo pipeline
# usage: launch_TEdenovo.py [ options ]
# options:
#      -h: this help
#      -P: project name
#      -C: configuration file (default=TEdenovo.cfg)
#      -S: step(s) (default='12345678', '1', '23', ...)"
#      -v: verbose (default=0/1/2/...)

import sys
import os
import time
from commons.core.utils.RepetOptionParser import RepetOptionParser

def main():
    
    description = "Launch TEdenovo pipeline"
    #TODO: examples
    examples = "EXAMPLE TODO"
    parser = RepetOptionParser(description = description, version = "v1.0", epilog = examples)
    parser.add_option( "-P", "--project", dest = "projectName", action="store", type="string", default = "", help = "project name (<=15 symbols, alphanumeric or underscore)")
    parser.add_option( "-C", "--config", dest = "configFileName", action="store", type="string", default = "TEdenovo.cfg", help = "configuration file (default = TEdenovo.cfg)" )
    parser.add_option( "-S", "--step", dest = "steps", action="store", type="string", default = "12345678", help = "step : 1/2/3/4/5/6/7/8 (default = 12345678)" )
    parser.add_option( "", "--struct", dest = "isStructuralSearch", action="store_true", default = False, help = "search for LTR using LTRharvest (default = False)")
    parser.add_option( "", "--onlyStruct", dest = "isOnlyStructuralSearch", action="store_true", default = False, help = "use only structural search (default = False)")
    parser.add_option( "-f", "--family", dest = "clustMethod", action="store", type="string", default = "", help = "program to cluster consensus to find consensus families (BlastClust/MCL)")
    parser.add_option( "-v", "--verbose", dest = "verbose", action="store", type="int", default = 0, help = "verbose (default = 0)")
    options = parser.parse_args()[0]    

    projectName = options.projectName
    configFileName = options.configFileName
    steps = options.steps
    clustMethod= options.clustMethod
    verbose = options.verbose
    isOnlyStructuralSearch = options.isOnlyStructuralSearch
    if isOnlyStructuralSearch:
        isStructuralSearch = True
    else:
        isStructuralSearch = options.isStructuralSearch
            
    if projectName == "":
        print "ERROR: missing project name (-P)"
        help()
        sys.exit(1)
        
    if configFileName == "":
        print "ERROR: missing configuration file (-C)"
        help()
        sys.exit(1)
        
    if not os.path.exists( configFileName ):
        print "ERROR: file '%s' doesn't exist" % configFileName
        help()
        sys.exit(1)
        
    if steps == "":
        print "ERROR: missing step(s) (-S)"
        help()
        sys.exit(1)
        
    print "START time: %s" % time.strftime("%Y-%m-%d %H:%M:%S")
    sys.stdout.flush()
    
    lCmds = []
    if "1" in steps:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 1 -v %i" % (projectName, configFileName, verbose) )
    if "2" in steps and not isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 2 -s Blaster -v %i" % (projectName, configFileName, verbose) )
    if "2" in steps and isStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 2 --struct -v %i" % (projectName, configFileName, verbose) )
    if "3" in steps and not isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 3 -s Blaster -c Grouper -v %i" % (projectName, configFileName, verbose) )
        lCmds.append( "TEdenovo.py -P %s -C %s -S 3 -s Blaster -c Recon -v %i" % (projectName, configFileName, verbose) )
        lCmds.append( "TEdenovo.py -P %s -C %s -S 3 -s Blaster -c Piler -v %i" % (projectName, configFileName, verbose) )
    if "3" in steps and isStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 3 --struct -v %i" % (projectName, configFileName, verbose) )
    if "4" in steps and not isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 4 -s Blaster -c Grouper -m Map -v %i" % (projectName, configFileName, verbose) )
        lCmds.append( "TEdenovo.py -P %s -C %s -S 4 -s Blaster -c Recon -m Map -v %i" % (projectName, configFileName, verbose) )
        lCmds.append( "TEdenovo.py -P %s -C %s -S 4 -s Blaster -c Piler -m Map -v %i" % (projectName, configFileName, verbose) )
    if "4" in steps and isStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 4 --struct -m Map -v %i" % (projectName, configFileName, verbose) )
        
    if "5" in steps and not isStructuralSearch: #Similarity search only
        lCmds.append( "TEdenovo.py -P %s -C %s -S 5 -s Blaster -c GrpRecPil -m Map -v %i" % (projectName, configFileName, verbose) )
    if "5" in steps and isOnlyStructuralSearch: #Structural search only
        lCmds.append( "TEdenovo.py -P %s -C %s -S 5 --struct -m Map -v %i" % (projectName, configFileName, verbose) )
    if "5" in steps and isStructuralSearch and not isOnlyStructuralSearch: #Both similarity and structural search
        lCmds.append( "TEdenovo.py -P %s -C %s -S 5 -s Blaster -c GrpRecPil --struct -m Map -v %i" % (projectName, configFileName, verbose) )

    if "6" in steps and not isStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 6 -s Blaster -c GrpRecPil -m Map -v %i" % (projectName, configFileName, verbose) )
    if "6" in steps and isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 6 --struct -m Map -v %i" % (projectName, configFileName, verbose) )
    if "6" in steps and isStructuralSearch and not isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 6 -s Blaster -c GrpRecPil --struct -m Map -v %i" % (projectName, configFileName, verbose) )
        
    if "7" in steps and not isStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 7 -s Blaster -c GrpRecPil -m Map -v %i" % (projectName, configFileName, verbose) )
    if "7" in steps and isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 7 --struct -m Map -v %i" % (projectName, configFileName, verbose) )
    if "7" in steps and isStructuralSearch and not isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 7 -s Blaster -c GrpRecPil --struct -m Map -v %i" % (projectName, configFileName, verbose) )

    if "8" in steps and not isStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 8 -s Blaster -c GrpRecPil -m Map -f %s -v %i" % (projectName, configFileName, clustMethod, verbose) )
    if "8" in steps and isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 8 --struct -m Map -f %s -v %i" % (projectName, configFileName, clustMethod, verbose) )
    if "8" in steps and isStructuralSearch and not isOnlyStructuralSearch:
        lCmds.append( "TEdenovo.py -P %s -C %s -S 8 -s Blaster -c GrpRecPil --struct -m Map -f %s -v %i" % (projectName, configFileName, clustMethod, verbose) )
        
    for cmd in lCmds:
        returnValue = os.system(cmd)
        if returnValue != 0:
            print "ERROR: command '%s' returned %i" % (cmd, returnValue)
            sys.exit(1)
            
    print "END time: %s" % time.strftime("%Y-%m-%d %H:%M:%S")
    sys.stdout.flush()
    
    return 0
    
if __name__== "__main__":
    main()
