import os,sys,copy,bisect

from pyRepet.coord.Range import *
from pyRepet.coord.Path import *

#------------------------------------------------------------------------------

class PathGraph:

    #--------------------------------------------------------------------------

    def __init__( self, file="" ):
        self.seq_linked=[]
        if file:
            self.to_log = file+'-'+str(os.getpid())+'-.log'
            self.log = open(self.to_log, 'a')
        else:
            self.to_log = ""

    #--------------------------------------------------------------------------

    class Node:

        def __init__(self):
            self.up = []
            self.down = []
            self.left = None
            self.right = None

            self.strand = None   # strand "+" or strand "-"
            self.NameSeq = ""
            self.NP = []         # Node's Paths

    #--------------------------------------------------------------------------

    # Parse a ".path" file to sort the HSPs and record them in a list

    def sort_HSP( self, pathFileName ):

        pathFile = open( pathFileName, 'r' )
        p = Path()
        lOrdHSP = []
        count = 0

        # for all the HSP recorded in the ".path" file
        while 1:
            if not p.read( pathFile ):
                break
            count += 1

            # don't keep the HSP when it is a match of one seq on itself
            if p.range_query.seqname == p.range_subject.seqname:
                continue

            # store the HSP with -1 * score !!
            my_tuple = ( -1*p.score, copy.deepcopy(p) )
            lOrdHSP.append( my_tuple )

        if count - len(lOrdHSP) > 0:
            print "\nignore the "+str(count - len(lOrdHSP))+\
                  " HSPs involving intra-sequence matchs"

        # sort the HSPs according to the score, in decreasing order,
        # the best scores are at the beginning
        lOrdHSP.sort()

        pathFile.close()

        return lOrdHSP

    # lOrdHSP = [(score_A,path_A),(score_B,path_B),...] with |scA| > |scB|

    #--------------------------------------------------------------------------

    def insert_seq( self, listNameSeq, seqA, seqB ):

        # seqA is already in listNameSeq but not seqB
        posA = listNameSeq.index(seqA) + 1

        middle = len(listNameSeq) / 2.0  

        # if seqA is in the 2nd part of the list
        if posA > middle:

            # seqB is inserted at the end of the list
            listNameSeq.append( seqB )

        # if seqA is in the 1st part of the list
        else :

            # seqB is inserted at the beginning
            listNameSeq = [seqB] + listNameSeq[0:]

        return listNameSeq

    #--------------------------------------------------------------------------

    def delete(self,liste,a,b):

        for i in liste:
            qn,sn=i.range_query.seqname,i.range_subject.seqname
            if (a,b)==(qn,sn) or (a,b)==(sn,qn):
                del liste[liste.index(i)]
            
    #--------------------------------------------------------------------------

    def new_limit(self,lns,limits,a,b,op):

        if op:
            if (lns.index(a)+1)>((len(lns)-1)/2.0):
                limits[1]=b
            else:
                limits[0]=b
        else:
            limits[limits.index(a)]=b      

    #--------------------------------------------------------------------------

    # Construct "lNameSeq" recording the "sequence-nodes" in vertical order

    def create_data_list( self, ordered_Path ):

        list_path = []
        lNameSeq = []
        limit_name = []

        # for each pair (score,Path), copy Path in list_path
        for i in ordered_Path:
            list_path.append( i[1] )    # i[1] is a Path instance

        # copy list_path in lOrdPath because list_path will be emptied 1 by 1
        lOrdPath = copy.deepcopy( list_path )    # lOrdPath=[path_A,path_B,...]

        first = 1
        sp = 0
        pos = 0

        # for all the paths recorded in list_path
        while len(list_path) > 0:

            i = list_path[pos]
            qn = i.range_query.seqname
            sn = i.range_subject.seqname

            if qn in limit_name or sn in limit_name or first:

                # if query not yet in lNameSeq
                if qn not in lNameSeq :

                    # if subject already in lNameSeq
                    if sn in lNameSeq :

                        # insert query according to position of subject
                        lNameSeq = self.insert_seq( lNameSeq, sn, qn )
                        self.new_limit( lNameSeq, limit_name, sn, qn, sp )
                        self.delete( list_path, qn, sn )
                        sp, first, pos = 0, 0, 0

                    # if both query and subject not yet in lNameSeq
                    else:

                        # insert them at the end of the list
                        lNameSeq.append( qn )
                        lNameSeq.append( sn )
                        if first == 1:
                            limit_name.append( qn )
                            limit_name.append( sn )
                        elif first == 2:
                            limit_name[1] = sn   # arbitrary
                        self.delete( list_path, qn, sn )
                        sp,first,pos=0,0,0

                # if subject not yet in lNameSeq but query is already
                elif sn not in lNameSeq :

                    # insert subject according to position of query
                    lNameSeq = self.insert_seq( lNameSeq, qn, sn )
                    self.new_limit( lNameSeq, limit_name, qn, sn, sp )
                    self.delete( list_path, qn, sn )
                    sp,first,pos=0,0,0
                else:
                    self.delete(list_path,qn,sn)
                    sp,first,pos=0,0,0

            else:
                if qn in lNameSeq or sn in lNameSeq:
                    first, sp = 1, 1
                elif (pos+1) <= (len(list_path)-1):
                    pos += 1
                else:
                    pos, first, sp = 0, 2, 1

        return lOrdPath, lNameSeq

    #--------------------------------------------------------------------------

    def vertic( self, lNameSeq, qryName, sbjName ):

        if lNameSeq.index( qryName ) > lNameSeq.index( sbjName ):
            vertical = 1    #  qn/sn

        else:
            vertical = -1   #  sn/qn

        return vertical

    #--------------------------------------------------------------------------

    # Create a list of primary nodes linked vertically

    def create_primary_nodes( self, lNameSeq ):

        lPrimNodes = []

        # create a new "Node" instance for each sequence
        for seq in lNameSeq:
            node = PathGraph.Node()
            node.NameSeq = seq
            lPrimNodes.append( node )

        # make the links "up" and "down" between each node
        for i in range(len(lPrimNodes)-1):
            lPrimNodes[i].up.append( lPrimNodes[i+1] )
            lPrimNodes[i+1].down.append( lPrimNodes[i] )

        return lPrimNodes    

    #--------------------------------------------------------------------------

    def no_overlap_side( self, nod, nodpath, rng, pos ):

        if not nodpath.overlap(rng):
            if nodpath.start>rng.end:
                print nodpath.start,rng.end
                if pos==-1:
                    if nod.up==[]:
                        return 1
                    if nod.up!=[]:
                        nb=len(nod.up)
                        for i in range(nb):
                            tn=nod.up[i]
                            for op in tn.NP:# op=other path that rng, op is included in tn.NP
                                oq=op.range_query.seqname 
                                os=op.range_subject.seqname
                                if rng.seqname==oq:
                                    if op.range_query.overlap(rng):
                                        print 'oook1'
                                        return 0
                                if rng.seqname==os:
                                    if op.range_subject.overlap(rng):
                                        print 'oook2'
                                        return 0
                    if nod.down!=[]:
                        nb=len(nod.down)
                        for i in range(nb):
                            tn=nod.down[i]
                            for op in tn.NP:# op=other path that rng, op include in tn.NP
                                oq=op.range_query.seqname
                                os=op.range_subject.seqname
                                if rng.seqname==oq:
                                    if op.range_query.overlap(rng) \
                                           and (op.range_subject.overlap(nod.NP[0].range_query) or op.range_subject.overlap(nod.NP[0].range_subject)):
                                        print 'oook3'
                                        return 2
                                if rng.seqname==os:
                                    if op.range_subject.overlap(rng) \
                                           and (op.range_subject.overlap(nod.NP[0].range_query) or op.range_subject.overlap(nod.NP[0].range_subject)):
                                        print 'oook4'
                                        return 2
                    return 1
                    
                if pos==1:
##                     eliminate=0
                    if nod.down==[]:
                        return 1
                    if nod.down!=[]:
                        nb=len(nod.down)
                        for i in range(nb):
                            tn=nod.down[i]
                            for op in tn.NP:# op=other path that rng, op include in tn.NP
                                oq=op.range_query.seqname
                                os=op.range_subject.seqname
                                if rng.seqname==oq:
                                    if op.range_query.overlap(rng):
                                        print 'okkk1'
                                        return 0
                                if rng.seqname==os:
                                    if op.range_subject.overlap(rng):
                                        print 'okkk2'
                                        return 0
                        #return 1 par defaut eliminate=0
                    if nod.up!=[]:
                        nb=len(nod.up)
                        for i in range(nb):
                            tn=nod.up[i]
                            for op in tn.NP:# op=other path that rng, op is included in tn.NP
                                oq=op.range_query.seqname 
                                os=op.range_subject.seqname
                                if rng.seqname==oq:
                                    if op.range_query.overlap(rng) \
                                           and (op.range_subject.overlap(nod.NP[0].range_query) or op.range_subject.overlap(nod.NP[0].range_subject)):
                                        print 'oook5'
                                        return 2
                                if rng.seqname==os:
                                    if op.range_subject.overlap(rng) \
                                           and (op.range_subject.overlap(nod.NP[0].range_query) or op.range_subject.overlap(nod.NP[0].range_subject)):
                                        print 'oook6'
                                        return 2
                    return 1
                    
            else:
                if pos==-1:
                    if nod.up==[]:
                        return -1
                    if nod.up!=[]:
                        nb=len(nod.up)                        
                        for i in range(nb):
                            tn=nod.up[i]
                            for op in tn.NP:# op=other path that rng, op is included in tn.NP
                                oq=op.range_query.seqname 
                                os=op.range_subject.seqname
                                if rng.seqname==oq:
                                    if op.range_query.overlap(rng):
                                        print 'oook7'
                                        return 0
                                if rng.seqname==os:
                                    if op.range_subject.overlap(rng):
                                        print 'oook8'
                                        return 0
                    if nod.down!=[]:
                        nb=len(nod.down)
                        for i in range(nb):
                            tn=nod.down[i]
                            for op in tn.NP:# op=other path that rng, op include in tn.NP
                                oq=op.range_query.seqname
                                os=op.range_subject.seqname
                                if rng.seqname==oq:
                                    if op.range_query.overlap(rng) \
                                           and (op.range_subject.overlap(nod.NP[0].range_query) or op.range_subject.overlap(nod.NP[0].range_subject)):
                                        print 'oook9'
                                        return 2
                                if rng.seqname==os:
                                    if op.range_subject.overlap(rng) \
                                           and (op.range_subject.overlap(nod.NP[0].range_query) or op.range_subject.overlap(nod.NP[0].range_subject)):
                                        print 'oook10'
                                        return 2
                    return -1 # right
                if pos==1:
                    if nod.down==[]:
                        return -1
                    if nod.down!=[]:
                        nb=len(nod.down)
                        for i in range(nb):
                            tn=nod.down[i]
                            for op in tn.NP:# op=other path that rng, op include in tn.NP
                                oq=op.range_query.seqname
                                os=op.range_subject.seqname
                                if rng.seqname==oq:
                                    if op.range_query.overlap(rng):
                                        print 'ok13'
                                        return 0
                                if rng.seqname==os:
                                    if op.range_subject.overlap(rng):
                                        print 'ok131'
                                        return 0
                    if nod.up!=[]:
                        print 'tictic',nodpath.start,nodpath.end,rng.start
                        nb=len(nod.up)
                        for i in range(nb):
                            tn=nod.up[i]
                            for op in tn.NP:# op=other path that rng, op is included in tn.NP
                                oq=op.range_query.seqname 
                                os=op.range_subject.seqname
                                if rng.seqname==oq:
                                    if op.range_query.overlap(rng) \
                                           and (op.range_subject.overlap(nod.NP[0].range_query) or op.range_subject.overlap(nod.NP[0].range_subject)):
                                        print 'here1'
                                        return 2
                                if rng.seqname==os:
                                    if op.range_subject.overlap(rng) \
                                           and (op.range_subject.overlap(nod.NP[0].range_query) or op.range_subject.overlap(nod.NP[0].range_subject)):
                                        print 'here2'
                                        return 2
                    return -1 # right
        elif pos==-1: # ==>nodpath.overlap(rng)=1
            tnp_nod=nod
            val=0
            if nod.up==[]:
                if tnp_nod.left!=None:
                    tnp_nod=tnp_nod.left
                    val=-1
                    if tnp_nod.up==[]:
                        pass
                    if tnp_nod.up!=[]:
                        for i in tnp_nod.NP:
                            if i.range_query.overlap(rng) or i.range_subject.overlap(rng):
                                print 'ok14'
                                return 0
                if val==-1:
                    tnp_nod=tnp_nod.right
                if tnp_nod.right!=None:
                    tnp_nod=tnp_nod.right
                    if tnp_nod.up==[]:
                        pass
                    if tnp_nod.up!=[]:
                        for i in tnp_nod.NP:
                            if i.range_query.overlap(rng) or i.range_subject.overlap(rng):
                                print 'ok15'
                                return 0
                return 2                    
            if nod.up!=[]:
                nb=len(nod.up)
                for i in range(nb):
                    tn=nod.up[i]
                    for op in tn.NP:# op=other path that rng, op is included in tn.NP
                        oq=op.range_query.seqname 
                        os=op.range_subject.seqname
                        if rng.seqname==oq:
                            if op.range_query.overlap(rng):
                                print 'ok16'
                                return 0
                        if rng.seqname==os:
                            if op.range_subject.overlap(rng):
                                print 'ok17'
                                return 0
                            
                if tnp_nod.left!=None:
                    tnp_nod=tnp_nod.left
                    val=-1
                    if tnp_nod.up==[]:
                        pass
                    if tnp_nod.up!=[]:
                        for i in tnp_nod.NP:
                            if i.range_query.overlap(rng) or i.range_subject.overlap(rng):
                                print 'ok18'
                                return 0
                if val==-1:
                    tnp_nod=tnp_nod.right
                if tnp_nod.right!=None:
                    tnp_nod=tnp_nod.right
                    if tnp_nod.up==[]:
                        pass
                    if tnp_nod.up!=[]:
                        for i in tnp_nod.NP:
                            if i.range_query.overlap(rng) or i.range_subject.overlap(rng):
                                print 'ok19'
                                return 0
                return 2
           
        elif pos==1: # ==>nodpath.overlap(rng)=1
            tnp_nod=nod
            val=0
            if nod.down==[]:
                if tnp_nod.left!=None:
                    tnp_nod=tnp_nod.left
                    val=-1
                    if tnp_nod.down==[]:
                        pass
                    if tnp_nod.down!=[]:
                        for i in tnp_nod.NP:
                            if i.range_query.overlap(rng) or i.range_subject.overlap(rng):
                                print 'ok1'
                                return 0
                if val==-1:
                    tnp_nod=tnp_nod.right
                if tnp_nod.right!=None:
                    tnp_nod=tnp_nod.right
                    if tnp_nod.down==[]:
                        pass
                    if tnp_nod.down!=[]:
                        for i in tnp_nod.NP:
                            if i.range_query.overlap(rng) or i.range_subject.overlap(rng):
                                print 'ok2'
                                return 0
                return 2
            elif nod.down!=[]:
                nb=len(nod.down)
                for i in range(nb):
                    tn=nod.down[i]
                    for op in tn.NP:# op=other path that rng, op include in tn.NP
                        oq=op.range_query.seqname 
                        os=op.range_subject.seqname
                        if rng.seqname==oq:
                            if op.range_query.overlap(rng):
                                print 'ok3'
                                return 0
                        if rng.seqname==os:
                            if op.range_subject.overlap(rng):
                                print 'ok4'
                                return 0
                            
                if tnp_nod.left!=None:
                    tnp_nod=tnp_nod.left
                    val=-1
                    if tnp_nod.down==[]:
                        pass
                    if tnp_nod.down!=[]:
                        for i in tnp_nod.NP:
                            if i.range_query.overlap(rng) or i.range_subject.overlap(rng):
                                print 'ok5'
                                return 0
                if val==-1:
                    tnp_nod=tnp_nod.right
                if tnp_nod.right!=None:
                    tnp_nod=tnp_nod.right
                    if tnp_nod.down==[]:
                        pass
                    if tnp_nod.down!=[]:
                        for i in tnp_nod.NP:
                            if i.range_query.overlap(rng) or i.range_subject.overlap(rng):
                                print 'ok6'
                                return 0
                print 'oki'
                return 2

    #--------------------------------------------------------------------------

    def crosskiller(self,X1,X2,Y1,Y2,Vertic,v,w,CurrentPath):

        if (X1,X2)==(1,1):
            Y1=-1
            return Y1
        if Y1==0 or Y2==0:
            Y1=0
            return Y1
        if Vertic==1:
            Y1=self.crossingParam(Y1,Y2,v,w,CurrentPath)
            return Y1
##             if (Y1,Y2)==(-1,-1):
##                 Y1=self.crossingParam(Y1,Y2,v,w,CurrentPath)
##                 return Y1
##             if (Y1,Y2)==(-1,1) or (Y1,Y2)==(-1,2):
##                 Y1=self.crossingParam(Y1,Y2,v,w,CurrentPath)
##                 return Y1
##             if (Y1,Y2)==(1,-1) or (Y1,Y2)==(2,-1):
##                 Y1=self.crossingParam(Y1,Y2,v,w,CurrentPath)
##                 return Y1
##             if (Y1,Y2)==(1,1) or (Y1,Y2)==(2,2) :
##                 Y1=self.crossingParam(Y1,Y2,v,w,CurrentPath)
##                 return Y1
##             if (Y1,Y2)==(2,1):
##                 Y1=self.crossingParam(Y1,Y2,v,w,CurrentPath)
##                 return Y1
##             if (Y1,Y2)==(1,2):
##                 Y1=self.crossingParam(Y1,Y2,v,w,CurrentPath)
##                 return Y1
        else: #Vertic==-1
            Y2=self.crossingParam(Y2,Y1,w,v,CurrentPath)
            return Y2
##             if (Y1,Y2)==(-1,-1):
##                 Y2=self.crossingParam(Y2,Y1,w,v,CurrentPath)
##                 return Y2
##             if (Y1,Y2)==(-1,1) or (Y1,Y2)==(-1,2):
##                 Y2=self.crossingParam(Y2,Y1,w,v,CurrentPath)
##                 return Y2
##             if (Y1,Y2)==(1,-1) or (Y1,Y2)==(2,-1):
##                 Y2=self.crossingParam(Y2,Y1,w,v,CurrentPath)
##                 return Y2
##             if (Y1,Y2)==(1,1) or (Y1,Y2)==(2,2):
##                 Y2=self.crossingParam(Y2,Y1,w,v,CurrentPath)
##                 return Y2
##             if (Y1,Y2)==(2,1):
##                 Y2=self.crossingParam(Y2,Y1,w,v,CurrentPath)
##                 return Y2
##             if (Y1,Y2)==(1,2):
##                 Y2=self.crossingParam(Y2,Y1,w,v,CurrentPath)
##                 return Y2
    
    #--------------------------------------------------------------------------

    def stop_or_continue(self,eye,stop,to=None):

        if to=='right':
            if eye.right!=None:
                eye=eye.right
                return eye,stop
            else:
                stop=0
                return eye,stop
        elif to=='left':
            if eye.left!=None:
                eye=eye.left
                return eye,stop
            else:
                stop=0
                return eye,stop
            
    #--------------------------------------------------------------------------

    def get_suspects_paths(self,suspect,Nod):

        for i in range(len(Nod.NP)):
            suspect.append(Nod.NP[i])
        return suspect
    
    #--------------------------------------------------------------------------

    def twisted(self,currentpath):

        if currentpath.range_query.getStrand()==currentpath.range_subject.getStrand():
            return 0 # direct
        else:
            return 1 # inverse
    
    #--------------------------------------------------------------------------

    def compare_lists_of_suspects_paths(self,list_A,list_B,y,currentpath,search_type):

        if len(list_A)!=0 and len(list_B)!=0:
            if search_type and self.twisted(currentpath): # search_type 1:cross, 0:parallele
                for path in list_A:
                    if path in list_B:
                        if not self.twisted(path):
                            y=0
                return y
            elif not search_type and not self.twisted(currentpath):
                for path in list_A:
                    if path in list_B:
                        if self.twisted(path):
                            y=0
                return y
            else:
                for path in list_A:
                    if path in list_B:
                        y=0
                return y
        else:
            return y

    #--------------------------------------------------------------------------

    def detect_intra_nod(self,y1,y2,v,w,Currentpath):

        ## capture des paths qui lient les deux seqs du path a introduire
        ## on ordonne pour chaque seq les start 
        ## on determine la positions que devrait occuper les 2 starts(query et subject)
        ## si positions egale ok sinon on invalide ce path
        eye_q=v
        eye_s=w
        wild_paths=[]
        wild_ranges_startA,wild_ranges_startB=[],[]
        if (y1,y2)==(2,1) or (y1,y2)==(1,2) or (y1,y2)==(2,-1) or (y1,y2)==(-1,2) or (y1,y2)==(2,2):
            for path in eye_q.NP:
                if ((self.twisted(Currentpath) and not self.twisted(path)) or (not self.twisted(Currentpath) and self.twisted(path)))\
                   and ((path.range_query.seqname==eye_q.NameSeq and path.range_subject.seqname==eye_s.NameSeq) \
                        or (path.range_query.seqname==eye_s.NameSeq and path.range_subject.seqname==eye_q.NameSeq)):
                    y1=0
                    return y1
                if path not in wild_paths:
                    wild_paths.append(path)
                    if (path.range_query.seqname==eye_q.NameSeq and path.range_subject.seqname==eye_s.NameSeq):
                        bisect.insort(wild_ranges_startA,(path.range_query.start))
                        bisect.insort(wild_ranges_startB,(path.range_subject.start))
                    if (path.range_query.seqname==eye_s.NameSeq and path.range_subject.seqname==eye_q.NameSeq):
                        bisect.insort(wild_ranges_startB,(path.range_query.start))
                        bisect.insort(wild_ranges_startA,(path.range_subject.start))
            for path in eye_s.NP:
                if ((self.twisted(Currentpath) and not self.twisted(path)) or (not self.twisted(Currentpath) and self.twisted(path))) \
                   and ((path.range_query.seqname==eye_q.NameSeq and path.range_subject.seqname==eye_s.NameSeq) \
                        or (path.range_query.seqname==eye_s.NameSeq and path.range_subject.seqname==eye_q.NameSeq)):
                    y1=0
                    return y1                    
                if path not in wild_paths:
                    wild_paths.append(path)
                    if (path.range_query.seqname==eye_q.NameSeq and path.range_subject.seqname==eye_s.NameSeq):
                        bisect.insort(wild_ranges_startA,(path.range_query.start))
                        bisect.insort(wild_ranges_startB,(path.range_subject.start))
                    if (path.range_query.seqname==eye_s.NameSeq and path.range_subject.seqname==eye_q.NameSeq):
                        bisect.insort(wild_ranges_startB,(path.range_query.start))
                        bisect.insort(wild_ranges_startA,(path.range_subject.start))
                        
            print 'r',wild_ranges_startA,wild_ranges_startB
            if Currentpath.range_query.seqname==eye_q.NameSeq:
                pos1=bisect.bisect(wild_ranges_startA,Currentpath.range_query.start)
                pos2=bisect.bisect(wild_ranges_startB,Currentpath.range_subject.start)
            if Currentpath.range_query.seqname==eye_s.NameSeq:
                pos1=bisect.bisect(wild_ranges_startB,Currentpath.range_query.start)
                pos2=bisect.bisect(wild_ranges_startA,Currentpath.range_subject.start)

            if self.twisted(Currentpath) and (len(wild_ranges_startA)==len(wild_ranges_startB)):
                pos1=len(wild_ranges_startA)-pos1  # len(wild_ranges_startA) et len(wild_ranges_startB) doivent normalement  etre egaux
                
            if pos1!=pos2:
                print 'p1',pos1,'p2',pos2
                y1=0
                return y1
        return y1
        
    #--------------------------------------------------------------------------

    def crossingParam(self,y1,y2,v,w,CurrentPath):

        parallele=0#       search_type
        cross=1   #
        
        eye_q=v
        eye_s=w
        
        if (y1,y2)==(-1,-1):
            suspect_path_q_A=[]
            suspect_path_s_B=[]
            stop_q,stop_s=1,1
            
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                else:
                    suspect_path_q_A=self.get_suspects_paths(suspect_path_q_A,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                else:
                    suspect_path_s_B=self.get_suspects_paths(suspect_path_s_B,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_B,y1,CurrentPath,parallele)
            return y1
        
        if (y1,y2)==(-1,1) or (y1,y2)==(-1,2):
            suspect_path_q_A=[]
            suspect_path_s_B=[]
            suspect_path_s_D=[]
            stop_q,stop_s=1,1

            if not self.detect_intra_nod(y1,y2,v,w,CurrentPath):
                y1=0
                return y1
            
            if (y1,y2)==(-1,2):            
                if eye_s.right!=None:
                    eye_s=eye_s.right
                else:
                    stop_s=0
                    
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                else:
                    suspect_path_q_A=self.get_suspects_paths(suspect_path_q_A,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                    
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'right')
                else:
                    suspect_path_s_D=self.get_suspects_paths(suspect_path_s_D,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'right')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_D,y1,CurrentPath,cross)
            if y1==0:
                return y1
        
            eye_s=w
            stop_q,stop_s=1,1
            eye_s=eye_s.left
            
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                else:
                    suspect_path_s_B=self.get_suspects_paths(suspect_path_s_B,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_B,y1,CurrentPath,parallele)
            return y1
        
        if (y1,y2)==(1,-1) or (y1,y2)==(2,-1):
            suspect_path_q_C=[]
            suspect_path_s_B=[]
            suspect_path_q_A=[]
            stop_q,stop_s=1,1
            
            if not self.detect_intra_nod(y1,y2,v,w,CurrentPath):
                y1=0
                return y1
            
            if (y1,y2)==(2,-1):       
                if eye_q.right!=None:
                    eye_q=eye_q.right
                else:
                    stop_q=0
                    
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'right')
                else:
                    suspect_path_q_C=self.get_suspects_paths(suspect_path_q_C,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'right')
                
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                else:
                    suspect_path_s_B=self.get_suspects_paths(suspect_path_s_B,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_C,suspect_path_s_B,y1,CurrentPath,cross)
            if y1==0:
                return y1

            eye_q=v
            stop_q,stop_s=1,1
            eye_q=eye_q.left
            
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                else:
                    suspect_path_q_A=self.get_suspects_paths(suspect_path_q_A,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_B,y1,CurrentPath,parallele)
            return y1 
        
        if (y1,y2)==(1,1) or (y1,y2)==(2,2):
            suspect_path_q_A=[]
            suspect_path_q_C=[]
            suspect_path_s_B=[]
            suspect_path_s_D=[]
            stop_q,stop_s=1,1
            
            if not self.detect_intra_nod(y1,y2,v,w,CurrentPath):
                y1=0
                return y1
            
            if (y1,y2)==(2,2):
                if eye_q.right!=None:
                    eye_q=eye_q.right
                else:
                    stop_q=0
                    
                eye_s=eye_s.left
            
            elif (y1,y2)==(1,1):
                eye_s=eye_s.left
                
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'right')
                else:
                    suspect_path_q_C=self.get_suspects_paths(suspect_path_q_C,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'right')
                    
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                else:
                    suspect_path_s_B=self.get_suspects_paths(suspect_path_s_B,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_C,suspect_path_s_B,y1,CurrentPath,cross)
            if y1==0:
                return y1
            
            eye_q=v
            eye_s=w
            stop_q,stop_s=1,1
            if (y1,y2)==(2,2):
                eye_q=eye_q.left
                
                if eye_s.right!=None:
                    eye_s=eye_s.right
                else:
                    stop_s=0
                    
            elif (y1,y2)==(1,1):
                eye_q=eye_q.left
                
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                else:
                    suspect_path_q_A=self.get_suspects_paths(suspect_path_q_A,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                        
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'right')
                else:
                    suspect_path_s_D=self.get_suspects_paths(suspect_path_s_D,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'right')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_D,y1,CurrentPath,cross)
            if y1==0:
                return y1
            
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_B,y1,CurrentPath,parallele)
            if y1==0:
                return y1
        
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_C,suspect_path_s_D,y1,CurrentPath,parallele)
            return y1
       
        if (y1,y2)==(2,1):
            suspect_path_q_A=[]
            suspect_path_s_B=[]
            suspect_path_q_C=[]
            suspect_path_s_D=[]
            stop_q,stop_s=1,1

            if not self.detect_intra_nod(y1,y2,v,w,CurrentPath):
                y1=0
                return y1
            
            if eye_q.right!=None:
                eye_q=eye_q.right
            else:
                stop_q=0
                
            eye_s=eye_s.left
                
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'right')
                else:
                    suspect_path_q_C=self.get_suspects_paths(suspect_path_q_C,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'right')
                        
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                else:
                    suspect_path_s_B=self.get_suspects_paths(suspect_path_s_B,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_C,suspect_path_s_B,y1,CurrentPath,cross)
            if y1==0:
                return y1
            
            eye_q=v
            eye_s=w
            stop_q,stop_s=1,1
            
            eye_q=eye_q.left
            
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')           
                else:
                    suspect_path_q_A=self.get_suspects_paths(suspect_path_q_A,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                    
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'right')
                else:
                    suspect_path_s_D=self.get_suspects_paths(suspect_path_s_D,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'right')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_D,y1,CurrentPath,cross)
            if y1==0:
                return y1
            
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_B,y1,CurrentPath,parallele)
            if y1==0:
                return y1
            
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_C,suspect_path_s_D,y1,CurrentPath,parallele)
            return y1
        
        if (y1,y2)==(1,2):
            suspect_path_q_A=[]
            suspect_path_s_B=[]
            suspect_path_q_C=[]
            suspect_path_s_D=[]
            stop_q,stop_s=1,1

            if not self.detect_intra_nod(y1,y2,v,w,CurrentPath):
                y1=0
                return y1
            
            eye_s=eye_s.left
            
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'right')
                else:
                    suspect_path_q_C=self.get_suspects_paths(suspect_path_q_C,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'right')
                        
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                else:
                    suspect_path_s_B=self.get_suspects_paths(suspect_path_s_B,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'left')
                    
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_C,suspect_path_s_B,y1,CurrentPath,cross)
            if y1==0:
                return y1
            
            eye_q=v
            eye_s=w
            stop_q,stop_s=1,1
            
            eye_q=eye_q.left
            
            if eye_s.right!=None:
                eye_s=eye_s.right
            else:
                stop_s=0
                
            while stop_q:
                if eye_q.down==[]:
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                else:
                    suspect_path_q_A=self.get_suspects_paths(suspect_path_q_A,eye_q)
                    eye_q,stop_q=self.stop_or_continue(eye_q,stop_q,'left')
                    
            while stop_s:
                if eye_s.up==[]:
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'right')
                else:
                    suspect_path_s_D=self.get_suspects_paths(suspect_path_s_D,eye_s)
                    eye_s,stop_s=self.stop_or_continue(eye_s,stop_s,'right')
                
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_D,y1,CurrentPath,cross)
            if y1==0:
                return y1
            
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_A,suspect_path_s_B,y1,CurrentPath,parallele)
            if y1==0:
                return y1
            
            y1=self.compare_lists_of_suspects_paths(suspect_path_q_C,suspect_path_s_D,y1,CurrentPath,parallele)
            return y1

    #--------------------------------------------------------------------------

    def horizontal_linking_type1(self,Node,reference):

        Node.left=reference
        reference.right=Node

    #--------------------------------------------------------------------------

    def horizontal_linking_type2(self,Node,reference):

        Node.right=reference
        reference=reference.left
        Node.left=reference
        reference.right=Node
        reference=reference.right
        reference=reference.right
        reference.left=Node
        
    #--------------------------------------------------------------------------

    def vertical_linking(self,Node_A,Node_B,vertical):

        if vertical==1:
            Node_A.down.append(Node_B)
            Node_B.up.append(Node_A)
        else:
            Node_B.down.append(Node_A)
            Node_A.up.append(Node_B)
            
    #--------------------------------------------------------------------------

    def add_Name_And_path(self,Node_A,Node_B,Ref_1,Ref_2,PATH):

        Node_A.NP.append(PATH)
        Node_B.NP.append(PATH)
        Node_A.NameSeq=Ref_1.NameSeq
        Node_B.NameSeq=Ref_2.NameSeq
        
    #--------------------------------------------------------------------------

    def linking_and_or_naming_a_node( self, Linking_typeA, Linking_typeB,\
                                      ref_v, ref_w, VERTIC, PATH ):

        if Linking_typeA == 1 and Linking_typeB == 0:
            newNod_A = PathGraph.Node()
            self.horizontal_linking_type1(newNod_A,ref_v)
            self.vertical_linking(newNod_A,ref_w,VERTIC)
            self.add_Name_And_path(newNod_A,ref_w,ref_v,ref_w,PATH)
            
        elif Linking_typeA == 1 and Linking_typeB == 1:
            newNod_A = PathGraph.Node()
            newNod_B = PathGraph.Node()
            self.horizontal_linking_type1(newNod_A,ref_v)
            self.horizontal_linking_type1(newNod_B,ref_w)
            self.vertical_linking(newNod_A,newNod_B,VERTIC)
            self.add_Name_And_path(newNod_A,newNod_B,ref_v,ref_w,PATH)

        elif Linking_typeA == 1 and Linking_typeB == 2:
            newNod_A = PathGraph.Node()
            newNod_B = PathGraph.Node()
            self.horizontal_linking_type1(newNod_A,ref_v)
            self.horizontal_linking_type2(newNod_B,ref_w)
            self.vertical_linking(newNod_A,newNod_B,VERTIC)
            self.add_Name_And_path(newNod_A,newNod_B,ref_v,ref_w,PATH)
        
        elif Linking_typeA == 2 and Linking_typeB == 0:
            newNod_A = PathGraph.Node()
            self.horizontal_linking_type2(newNod_A,ref_v)
            self.vertical_linking(newNod_A,ref_w,VERTIC)
            self.add_Name_And_path(newNod_A,ref_w,ref_v,ref_w,PATH)
            
        elif Linking_typeA == 2 and Linking_typeB == 1:
            newNod_A = PathGraph.Node()
            newNod_B = PathGraph.Node()
            self.horizontal_linking_type2(newNod_A,ref_v)
            self.horizontal_linking_type1(newNod_B,ref_w)
            self.vertical_linking(newNod_A,newNod_B,VERTIC)
            self.add_Name_And_path(newNod_A,newNod_B,ref_v,ref_w,PATH)
            
        elif Linking_typeA == 2 and Linking_typeB == 2:
            newNod_A = PathGraph.Node()
            newNod_B = PathGraph.Node()
            self.horizontal_linking_type2(newNod_A,ref_v)
            self.horizontal_linking_type2(newNod_B,ref_w)
            self.vertical_linking(newNod_A,newNod_B,VERTIC)
            self.add_Name_And_path(newNod_A,newNod_B,ref_v,ref_w,PATH)
            
        elif Linking_typeA == 0 and Linking_typeB == 0:
            self.vertical_linking(ref_v,ref_w,VERTIC)
            self.add_Name_And_path(ref_v,ref_w,ref_v,ref_w,PATH)
            
        elif Linking_typeA == 0 and Linking_typeB == 1:
            newNod_B = PathGraph.Node()
            self.horizontal_linking_type1(newNod_B,ref_w)
            self.vertical_linking(ref_v,newNod_B,VERTIC)
            self.add_Name_And_path(ref_v,newNod_B,ref_v,ref_w,PATH)
            
        elif Linking_typeA == 0 and Linking_typeB == 2:
            newNod_B = PathGraph.Node()
            self.horizontal_linking_type2(newNod_B,ref_w)
            self.vertical_linking(ref_v,newNod_B,VERTIC)
            self.add_Name_And_path(ref_v,newNod_B,ref_v,ref_w,PATH)
            
    #--------------------------------------------------------------------------

    def insert_node( self, y1, x1, y2, x2, vertic, V, dbV, Path ):

        if y1 == 0 or y2 == 0:
            return

        if x1 == 1:
            if x2 == 1:  #1
                self.linking_and_or_naming_a_node(1,1,V,dbV,vertic,Path)
                
            elif x2 == 0 and y2 == 1:  #2
                self.linking_and_or_naming_a_node(1,2,V,dbV,vertic,Path)
       
            elif x2 == 0 and y2 == -1:  #3
                self.linking_and_or_naming_a_node(1,1,V,dbV,vertic,Path)

            elif x2 == 0 and y2 == 2:  #4
                self.linking_and_or_naming_a_node(1,0,V,dbV,vertic,Path)

        elif x1 == 0:
            if x2 == 1 and y1 == 1:  #5
                self.linking_and_or_naming_a_node(2,1,V,dbV,vertic,Path)

            elif x2 == 1 and y1 == -1:  #6
                self.linking_and_or_naming_a_node(1,1,V,dbV,vertic,Path)

            elif x2 == 1 and y1 == 2:  #7
                self.linking_and_or_naming_a_node(0,1,V,dbV,vertic,Path)

            elif x2 == 0 and y1 == 1 and y2 == 1:  #8
                self.linking_and_or_naming_a_node(2,2,V,dbV,vertic,Path)
               
            elif x2 == 0 and y1 == 1 and y2 == -1:  #9
                self.linking_and_or_naming_a_node(2,1,V,dbV,vertic,Path)
               
            elif x2 == 0 and y1 == 1 and y2 == 2:  #10
                self.linking_and_or_naming_a_node(2,0,V,dbV,vertic,Path)

            elif x2 == 0 and y1 == -1 and y2 == 1:  #11
                self.linking_and_or_naming_a_node(1,2,V,dbV,vertic,Path)
                
            elif x2 == 0 and y1 == -1 and y2 == -1:  #12
                self.linking_and_or_naming_a_node(1,1,V,dbV,vertic,Path)
               
            elif x2 == 0 and y1 == -1 and y2 == 2:  #13
                self.linking_and_or_naming_a_node(1,0,V,dbV,vertic,Path)

            elif x2 == 0 and y1 == 2 and y2 == 1:  #14
                self.linking_and_or_naming_a_node(0,2,V,dbV,vertic,Path)
           
            elif x2 == 0 and y1 == 2 and y2 == -1:  #15
                self.linking_and_or_naming_a_node(0,1,V,dbV,vertic,Path)
                
            elif x2 == 0 and y1 == 2 and y2 == 2:  #16
                self.linking_and_or_naming_a_node(0,0,V,dbV,vertic,Path)

    #--------------------------------------------------------------------------

    def update_seq_linked(self,data,path):

        tmp=[]
        indexQuery,indexSubject=None,None
        for list in data:
            if path.range_query.seqname in list:
                indexQuery=data.index(list)
                break
        for list in data:
            if path.range_subject.seqname in list:
                indexSubject=data.index(list)
                break
        if indexQuery==indexSubject and indexQuery!=None :
            return data[indexQuery]
        elif indexQuery==None and indexSubject==None:
            tmp.append(path.range_query.seqname)            
            tmp.append(path.range_subject.seqname)
            data.append(tmp)
            return tmp
        elif indexQuery!=None and indexSubject!=None:
            data[indexQuery].extend(data[indexSubject])
            tmp=data[indexQuery]
            data.remove(data[indexSubject])
            return tmp
        elif indexQuery!=None and indexSubject==None:
            data[indexQuery].append(path.range_subject.seqname)
            return data[indexQuery]
        elif indexQuery==None and indexSubject!=None:
            data[indexSubject].append(path.range_query.seqname)
            return data[indexSubject]
            
    #--------------------------------------------------------------------------

    def choice_strand(self,Wsubject,Vquery,newlist,vertic,Y1,Y2,X1,X2,path):

        tmp=copy.deepcopy(self.seq_linked)
        seqlinked=self.update_seq_linked(self.seq_linked,path)
        
        query=path.range_query
        subject=path.range_subject

        if Y1==0 or Y2==0:
            pass
        
        elif (X1,X2)==(1,1):
            Vquery.strand=path.range_query.getStrand()
            Wsubject.strand=path.range_subject.getStrand()
            
        elif Vquery.strand==Wsubject.strand and query.getStrand()==subject.getStrand():
            pass
        
        elif Vquery.strand==Wsubject.strand and query.getStrand()!=subject.getStrand():
            oriV=Vquery
            oriW=Wsubject
            if vertic==1:
                while oriV.right!=None and Y1!=0:
                    oriV=oriV.right
                    if oriV.down!=[]:
                        Y1=0
                while oriW.right!=None and Y1!=0:
                    oriW=oriW.right
                    if oriW.up!=[]:
                        Y1=0
            else:  #vertic=-1
                while oriV.right!=None and Y1!=0:
                    oriV=oriV.right
                    if oriV.up!=[]:
                        Y1=0
                while oriW.right!=None and Y1!=0:
                    oriW=oriW.right
                    if oriW.down!=[]:
                        Y1=0
            if Y1:
                self.change_strand(newlist,vertic,query,subject,seqlinked)
                    
        elif (Vquery.strand==None and Wsubject!=None) and query.getStrand()==subject.getStrand():
            Vquery.strand=Wsubject.strand
        elif (Vquery.strand==None and Wsubject!=None) and query.getStrand()!=subject.getStrand():
            if Wsubject.strand=="+":
                Vquery.strand="-"
            elif Wsubject.strand=="-":
                Vquery.strand="+"

        elif (Wsubject.strand==None and Vquery!=None) and query.getStrand()==subject.getStrand():
            Wsubject.strand=Vquery.strand
        elif (Wsubject.strand==None and Vquery!=None) and query.getStrand()!=subject.getStrand():
            if Vquery.strand=="+":
                Wsubject.strand="-"
            elif Vquery.strand=="-":
                Wsubject.strand="+"

        elif Vquery.strand!=Wsubject.strand and query.getStrand()==subject.getStrand():
            oriV=Vquery
            oriW=Wsubject
            if vertic==1:
                while oriV.right!=None and Y1!=0:
                    oriV=oriV.right
                    if oriV.down!=[]:
                        Y1=0
                while oriW.right!=None and Y1!=0:
                    oriW=oriW.right
                    if oriW.up!=[]:
                        Y1=0
                    
            else:  #vertic=-1
                while oriV.right!=None and Y1!=0:
                    oriV=oriV.right
                    if oriV.up!=[]:
                        Y1=0
                while oriW.right!=None and Y1!=0:
                    oriW=oriW.right
                    if oriW.down!=[]:
                        Y1=0
            if Y1:
                self.change_strand(newlist,vertic,query,subject,seqlinked)

        elif Vquery.strand!=Wsubject.strand and query.getStrand()!=subject.getStrand():
            pass

        else:
            print 'Error in self.choice_strand()'
            sys.exit(0)
        
        if Y1==0 or Y2==0:
            self.seq_linked=copy.deepcopy(tmp)
        return Y1

    #--------------------------------------------------------------------------

    def change_strand(self,newlist,vertic,query,subject,seqlinked):

        if vertic==1:
            ori=newlist[0]
            while ori.NameSeq!=query.seqname:
                if ori.NameSeq in seqlinked:
                    if ori.strand=="+":
                        ori.strand="-"
                    elif ori.strand=="-":
                        ori.strand="+"
                ori=ori.up[0]
        elif vertic==-1:        
            ori=newlist[0]
            while ori.NameSeq!=subject.seqname:
                if ori.NameSeq in seqlinked:
                    if ori.strand=="+":
                        ori.strand="-"
                    elif ori.strand=="-":
                        ori.strand="+"
                ori=ori.up[0]

    #--------------------------------------------------------------------------

    def build_other_nodes( self, lOrdPath, lNameSeq, lNodes ):

        count = 0

        # for each HSP
        for path in lOrdPath:

            qn = path.range_query.seqname
            sn = path.range_subject.seqname
            Vertic = self.vertic( lNameSeq, qn, sn )

            #------------------------------------------------------------------

            # iterator for the "query" fragment
            v = lNodes[0]
            X1, Y1 = 0, -1

            # walk in lNodes to find the sequence from which
            # the query fragment comes
            while v.NameSeq != qn:
                if v.up == []:
                    print "*** Error v: seq not in lNodes ***"
                    return
                else:
                    v = v.up[0]
            Vquery = v

            # go to the node the most at the right and look if there is an
            # overlap between it and the HSP currently analysed
            if v.right == None:
                X1 = 1
            else:
                while v.right != None and Y1 != 0 and Y1 != 1 and Y1 != 2:
                    v = v.right
                    bigv = copy.copy(v)
                    if bigv.NP[0].range_query.seqname == qn:
                        Y1 = self.no_overlap_side( bigv, bigv.NP[0].range_query, path.range_query, Vertic )
                    else:
                        Y1 = self.no_overlap_side( bigv, bigv.NP[0].range_subject, path.range_query, Vertic )

            #------------------------------------------------------------------

            # iterator for the "subject" fragment
            w = lNodes[0]
            X2, Y2 = 0, -1

            while Y1 != 0 and w.NameSeq != sn:
                if w.up == []:
                    print "*** Error w: seq not in lNodes ***"
                    return
                else:
                    w = w.up[0]
            Wsubject = w

            if w.right == None and Y1 != 0:
                X2 = 1
            else:
                while w.right != None and Y2!=0 and Y2!=1 and Y2!=2 and Y1!=0:
                    w = w.right
                    bigw = copy.copy(w)
                    if bigw.NP[0].range_subject.seqname == sn:
                        Y2 = self.no_overlap_side( bigw, bigw.NP[0].range_subject, path.range_subject, Vertic*(-1) )
                    else:
                        Y2 = self.no_overlap_side( bigw, bigw.NP[0].range_query, path.range_subject, Vertic*(-1) )

            #------------------------------------------------------------------

            # according to the positional parameters of the HSP (qry + sbj),
            # modify Y1 or Y2 if the configuration has to be counter-selected
            if Vertic == 1:
                Y1 = self.crosskiller(X1,X2,Y1,Y2,Vertic,v,w,path)
            else:
                Y2 = self.crosskiller(X1,X2,Y1,Y2,Vertic,v,w,path)

            # before inserting the node, allocate a sign to the sequences
            Y1 = self.choice_strand(Wsubject,Vquery,lNodes,Vertic,Y1,Y2,X1,X2,path)

            # insert the new Node in the graph
            self.insert_node( Y1, X1, Y2, X2, Vertic, v, w, path )

            #------------------------------------------------------------------

            if Y1 != 0 and Y2 != 0:

                count += 1
                print "Y1",Y1,"Y2",Y2,"X1",X1,"X2",X2,"V",Vertic
                print "* Number",count,"->",path.id,path.score,path.identity,
                print ",",qn.split(" ",1)[0],path.range_query.getStrand(),
                print path.range_query.start,path.range_query.end,
                print ",",sn.split(" ",1)[0],path.range_subject.getStrand(),
                print path.range_subject.start,path.range_subject.end

                if self.to_log:
                    if count == 1:
                        self.log.write("\n")
                    output = "Y1 "+str(Y1)+" Y2 "+str(Y2)+" X1 "+str(X1)+" X2 "+str(X2)+" V "+str(Vertic)+"\n" \
                             +"Number "+str(count)+" -> "+str(path.id)+" "+str(path.score)+" "+str(path.identity) \
                             +" "+str(qn.split(" ",1)[0])+" "+path.range_query.getStrand()+" "+str(path.range_query.start)+" "+str(path.range_query.end) \
                             +" "+str(sn.split(" ",1)[0])+" "+path.range_subject.getStrand()+" "+str(path.range_subject.start)+" "+str(path.range_subject.end)+"\n"
                    self.log.write( output )

           ##  else:
##                 print "trash"
##                 count+=1
##                 print "Y1",Y1,"Y2",Y2,"X1",X1,"X2",X2,"V",Vertic
##                 print "Number",count,"->",path.id,path.score,path.identity,qn.split(" ",1)[0],
##                 print path.range_query.getStrand(),
##                 print path.range_query.start,path.range_query.end,sn.split(" ",1)[0],
##                 print path.range_subject.getStrand(),
##                 print path.range_subject.start,path.range_subject.end
        
##         light=lNodes[0]
##         light=light.up[0]
##         light=light.up[0]
##         light=light.up[0]
##         print "light.nam :",light.NameSeq
##         n=0
##         while light.right!=None:
##             light=light.right
##             for i in light.NP:
##                 print i.score
##             n+=1
##         print "nbre de noeuds :",n
                
        return lNodes
