import math
import re
from pyRepet.seq.Bioseq import *
from pyRepet.util.Stat import *


#---------------------------------------------------------------------
class InfoTE:
	TElength={}
	TEos={}
	TEkw={}
	TEde={}
	
	def __init__(self,fileseq,fileinfo):
		self.read_length(fileseq)
		self.read_info(fileinfo)
		
	def read_info(self,filename):
		file=open(filename)
		while 1:
			line=file.readline()
			if line=="":
				break
			line=line[:-1]
			tok=line.split("\t")
			self.TEos[tok[0]]=tok[2]
			self.TEkw[tok[0]]=tok[1]
			self.TEde[tok[0]]=tok[3]
		file.close()
		print "done!"
		
	def read_length(self,filename):
		file=open(filename)
		seq=Bioseq()
		while 1:
			seq.read(file)
			if seq.sequence==None:
				break
			name=(seq.header.split()[0])
			print "reading length...",name
			self.TElength[name]=seq.length()
		file.close()
		print "done!"
		
	def size(self):
		return len(self.TEtype)
#---------------------------------------------------------------------
class MatchWatch:
	def __init__(self,fileTEseq,fileTEinfo,fileseq,filematch):
		self.infoMatch={}
		self.infoId={}
		self.infoSeq={}
		self.infoTE=InfoTE(fileTEseq,fileTEinfo)
		self.read_seq(fileseq)
		self.read_match(filematch)

	def read_seq(self,filename):
		self.seq_filename=filename
		file=open(self.seq_filename)
		seq=Bioseq()
		while 1:
			seq.read(file)
			if seq.sequence==None:
				break
			self.infoSeq[seq.header]=seq.length()

	def read_match(self,filename):
		self.matcher_filename=filename
		file=open(self.matcher_filename)
		line=file.readline()
		while 1:
			line=file.readline()
			if line=="":
				break
			token=line.split('\t')
			seqname=token[0]
			seq_start=int(token[1])
			seq_end=int(token[2])
			matchname=token[6]
			matchlen=float(token[5])
			identity=float(token[13])
			if matchname not in self.infoMatch.keys():
				self.infoMatch[matchname]=Stat.Quantile()
				self.infoId[matchname]=Stat.Stat()
			self.infoMatch[matchname].add(matchlen)
			self.infoId[matchname].add(identity)

	def write(self,filename):
		file=open(filename,"w")

		l=[]
		for i in self.infoMatch.keys():
			str="%d\t%d\t%3.3f\t%3.3f\t%3.3f\t%3.3f\t%3.3f\t%3.3f"%\
			(self.infoTE.TElength[i],\
			 self.infoMatch[i].n,\
			 self.infoMatch[i].min,\
			 self.infoMatch[i].quantile(0.25),\
			 self.infoMatch[i].quantile(0.5),\
			 self.infoMatch[i].quantile(0.75),\
			 self.infoMatch[i].max,\
			 self.infoId[i].mean())
			 
			l.append((i,str))
		file.write("TE matches: %d\n"%len(l))
		file.write("-----------------\n")
		l.sort()
		file.write("name\tkw\tos\tlength\tnumber\tmin_match\tQ_match(0.25)\tmedian_match\tQ_match(0.75)\tmax_match\tmean_identity\n")
		for i in l:
			if self.infoTE.TEkw.has_key(i[0]):
				file.write("%-15s\t%s\t%s\t%s\n"%\
					   (i[0],self.infoTE.TEkw[i[0]],\
					    self.infoTE.TEos[i[0]],i[1]))
			else:
				file.write("%-15s\t?\tDrosophila melanogaster\t%s\n"%\
					   (i[0],i[1]))
				 
		l=[]
		for i in self.infoTE.TElength.keys():
			if not self.infoMatch.has_key(i):
				l.append(i)
		file.write("\nTE not found: %d\n"% len(l))
		file.write("-----------------------\n")
		l.sort()
		for i in l:
			file.write("%-15s\n"%(i))

		file.close()

	def write_tab(self,filename):
		file=open(filename,"w")

		l=[]
		for i in self.infoMatch.keys():
			str="%d\t%3.3f\t%3.3f\t%3.3f\t%3.3f\t%3.3f\t%3.3f"%\
			 (self.infoMatch[i].n,\
			 self.infoMatch[i].min,\
			 self.infoMatch[i].quantile(0.25),\
			 self.infoMatch[i].quantile(0.5),\
			 self.infoMatch[i].quantile(0.75),\
			 self.infoMatch[i].max,\
			 self.infoId[i].mean())
			 
			l.append((i,str))
		file.write("name\tnumber\tmin_match\tQ_match(0.25)\tmedian_match\tQ_match(0.75)\tmax_match\tmean_identity\n")
		for i in l:
			file.write("%-15s\t%s\n"%(i[0],i[1]))
		file.close()
		
	def write_distr_length_perc(self,filename):
		file=open(filename,"w")

		for i in self.infoMatch.keys():
			for j in self.infoMatch[i].values:
				file.write("%s\t%3.3f\n"%(i,j))
		
	def TE_found(self,infoTE):
		found=[]
		notfound=[]
		for i in infoTE.TElength.keys():
			if i in self.infoMatch.keys():
				found.append(i)
			else:
				notfound.append(i)
		return found,notfound
			
		
#---------------------------------------------------------------------
class GrouperWatch:
	clust2group={}
	group2clust={}
	group_sizes={}
	group_length={}
	grouper_filename=""
	nb_group=0
	nb_clust=0
	
	def read_grouper(self,filename):
		self.grouper_filename=filename
		file=open(self.grouper_filename)
		line="start"
		srchGrp=re.compile('Gr\d+')
		srchCl=re.compile('Cl\d+')
		seq=Bioseq()
		sum={}
		count={}
		while 1:
			seq.read(file)
			if seq.sequence==None:
				break
			m=srchGrp.search(seq.header)
			gr=int(m.string[m.start(0)+2:m.end(0)])	
			m=srchCl.search(seq.header)
			cl=int(m.string[m.start(0)+2:m.end(0)])
			if count.has_key(gr):
				count[gr]=count[gr]+1
				sum[gr]=sum[gr]+seq.length()
			else:
				count[gr]=1
				sum[gr]=seq.length()

			self.group2clust[gr]=cl
			if self.clust2group.has_key(cl):
				if gr not in self.clust2group[cl]:
					self.clust2group[cl].append(gr)
			else:
				self.clust2group[cl]=[gr]

		for i in count.keys():
			self.group_length[i]=sum[i]/count[i]
			self.group_sizes[i]=count[i]
		self.nb_group=len(self.group_sizes)
		self.nb_clust=len(self.clust2group)

	def extract_group(self,filename):
		file=open(filename)
		line="start"
		srchGrp=re.compile('Gr\d+')
		srchCl=re.compile('Cl\d+')
		seq=Bioseq()
		grouplist=[]
		while 1:
			seq.read(file)
			if seq.sequence==None:
				break
			m=srchGrp.search(seq.header)
			gr=int(m.string[m.start(0)+2:m.end(0)])	

			if gr not in grouplist:
				grouplist.append(gr)
		return grouplist

	def group_size(self,gr):
		return self.group_sizes[gr]

	def clust_size(self,cl):
		sum=0
		for i in self.clust2group[cl]:
			sum=sum+self.group_sizes[i]
		return sum
				
	def whereIsGroup(self,gr):
		for i in self.clust2group.keys():
			if gr in self.clust2group[i]:
				print "in cluster #",i
				
	def sort_group_sizes(self):
		vec=[]	
		for i in self.group_sizes.keys():
			vec.append((self.group_sizes[i],self.group_length[i],i))
		vec.sort()
		return vec

	def rev_sort_group_sizes(self):
		vec=[]	
		for i in self.group_sizes.keys():
			vec.append((self.group_sizes[i]*-1,self.group_length[i],i))
		vec.sort()
		vec2=[]	
		for i in vec:
			vec2.append((i[0]*-1,i[1],i[2]))
		return vec2

	def dist_group_sizes(self):
		h={}	
		for i in self.group_sizes.keys():
			size=self.group_sizes[i]
			if h.has_key(size):
				h[size]=h[size]+1
			else:
				h[size]=1
		vec=[]
		for i in h.keys():
			vec.append((i,h[i]))
		vec.sort()
		return vec

	def dist_clust_sizes(self):
		h={}	
		for i in self.clust2group.keys():
			size=self.clust_size(i)
			if h.has_key(size):
				h[size]=h[size]+1
			else:
				h[size]=1
		vec=[]
		for i in h.keys():
			vec.append((i,h[i]))
		vec.sort()
		return vec

	def group_length_size(self,filename):
		fileout=open(filename,'w')
		for i in self.group_length.keys():
			fileout.write(str(self.group_length[i])+"\t"+str(self.group_sizes[i]))
		fileout.close()

	def write_group_info(self,filename,grouplist=[]):
		l=self.rev_sort_group_sizes()
		fileout=open(filename,'w')
		fileout.write("Number of groups=%d\n"%self.nb_group)
		fileout.write("Number of clusters=%d\n"%self.nb_clust)
		for i in l:
			if i in grouplist or grouplist==[]:
				fileout.write("group:"+str(i[2])+\
					      " clust:"+str(self.group2clust[i[2]])+\
					      " size:"+str(i[0])+\
					      " length:"+str(i[1])+"\n")
		fileout.close()

	def write_group_sizes(self,filename):
		lg=self.dist_group_sizes()
		fileout=open(filename,'w')
		for i in lg:
			fileout.write("%d\t%d\n"%i)
		fileout.close()

	def write_clust_sizes(self,filename):
		lc=self.dist_clust_sizes()
		fileout=open(filename,'w')
		for i in lc:
			fileout.write("%d\t%d\n"%i)
		fileout.close()

		
#---------------------------------------------------------------------
class GrouperMatchWatch(GrouperWatch):
	match=[]
	group2match={}
	clust2match={}
	matcher_filename=""

	def read_match(self,filename):
		self.match=[]
		self.matcher_filename=filename
		file=open(self.matcher_filename)
		line="start"
		srchGrp=re.compile('Gr\d+')
		srchCl=re.compile('Cl\d+')
		sum={}
		count={}
		line=file.readline()
		while 1:
			line=file.readline()
			if line=="":
				break
			token=line.split('\t')
			seqname=token[6]
			if seqname not in self.match:
				self.match.append(seqname)
			m=srchGrp.search(token[0])
			gr=int(m.string[m.start(0)+2:m.end(0)])	
			m=srchCl.search(token[0])
			cl=int(m.string[m.start(0)+2:m.end(0)])
			if self.group2match.has_key(gr):
				if seqname not in self.group2match[gr]:
					self.group2match[gr].append(seqname)
			else:
				self.group2match[gr]=[seqname]
			if self.clust2match.has_key(cl):
				if seqname not in self.clust2match[cl]:
					self.clust2match[cl].append(seqname)
			else:
				self.clust2match[cl]=[seqname]


	def match2group(self,seq):
		g=[]
		for i in self.group2match.keys():
			if seq in self.group2match[i]:
				g.append(i)
		return g

	def match2clust(self,seq):
		g=[]
		for i in self.clust2match.keys():
			if seq in self.clust2match[i]:
				g.append(i)
		return g

	def TE_found(self,infoTE):
		found=[]
		notfound=[]
		for i in infoTE.TElength.keys():
			if i in self.match:
				found.append(i)
			else:
				notfound.append(i)
		return found,notfound

				
	def whereIsSeq(self,seq):
		for i in self.clust2match.keys():
			if seq in self.clust2match[i]:
				print "in cluster #",i,"group:",self.clust2group[i]
		for i in self.group2match.keys():
			if seq in self.group2match[i]:
				print "in group #",i
				
	def whoIsGroup(self,gr):
		if self.group2match.has_key(gr):
			print "group #",gr,"is",self.group2match[gr]
		else:
			print "group #",gr,"has no match"

	def whoIsCluster(self,cl):
		if self.clust2match.has_key(cl):
			print "cluster #",cl,"is",self.clust2match[cl]
			print "and contain groups",self.clust2group[cl]
		else:
			print "cluster #",cl,"has no match"

	def group_no_match(self):
		l=[]
		for i in self.group_sizes.keys():
			if i not in self.group2match.keys():
				l.append(i)
		return l

	def clust_no_match(self):
		l=[]
		for i in self.clust2group.keys():
			if i not in self.clust2match.keys():
				l.append(i)
		return l

	def write_group_nbmatchesXsize(self,filename):
		fileout=open(filename,'w')
		for i in self.group2match.keys():
			fileout.write(str(self.group_sizes[i])+"\t"+str(len(self.group2match[i]))+"\n")
		fileout.close()
		
	def write_group_nbmatchesXlength(self,filename):
		fileout=open(filename,'w')
		for i in self.group2match.keys():
			fileout.write(str(self.group_length[i])+"\t"+str(len(self.group2match[i]))+"\n")
		fileout.close()
		
	def groupsize_sort_matches(self):
		l=self.rev_sort_group_sizes()
		l2=[]
		for i in l:
			if self.group2match.has_key(i[2]):
				l2.append((i[2],i[0],i[1],self.group2match[i[2]]))
		return l2
	
	def one2one(self):
		l={}
		for i in self.match:
			lg=self.match2group(i)
			for j in lg:
				g=self.group2match[j]
				if len(g)==1 and g[0]==i:
					if l.has_key(i):
						l[i].append(j)
					else:
						l[i]=[j]
		return l

	def neverAloneMatches(self):
		l=self.one2one()
		m=[]
		m.extend(self.match)
		for i in l.keys():
			m.remove(i)
		m.sort()
		return m

	def match2nbGroup(self):
		l=[]
		for i in self.match:
			g=self.match2group(i)
			l.append((i,len(g),g))
		l.sort()
		return l
	
	def dist_match_group(self):
		h={}
		for i in self.group2match.keys():
			nb_match=len(self.group2match[i])
			if h.has_key(nb_match):
				h[nb_match]=h[nb_match]+1
			else:
				h[nb_match]=1
		return h

	def dist_match_clust(self):
		h={}
		for i in self.clust2match.keys():
			nb_match=len(self.clust2match[i])
			if h.has_key(nb_match):
				h[nb_match]=h[nb_match]+1
			else:
				h[nb_match]=1
		return h

	def write_dist_match(self,filename):
		fileout=open(filename+".dist_match_group.dat",'w')
		h=self.dist_match_group()
		l=h.keys()
		l.sort()
		for i in l:
			fileout.write("%d\t%d\n"%(i,h[i]))
		fileout.close()
		
		fileout=open(filename+".dist_match_clust.dat",'w')
		h=self.dist_match_clust()
		l=h.keys()
		l.sort()
		for i in l:
			fileout.write("%d\t%d\n"%(i,h[i]))
		fileout.close()

	def splitted_group(self):
		sp=[]
		for i in self.match:
			l=self.match2group(i)
			print i, l
			if len(l)>1 :
				sp.append((i,l))
		return sp
				
	def splitted_group_dist(self):
		h={}
		for i in self.match:
			l=self.match2group(i)
			print i, l
			nb_group=len(l)
			if h.has_key(nb_group):
				h[nb_group]=h[nb_group]+1
			else:
				h[nb_group]=1
		return h
				
	def splitted_clust(self):
		sp=[]
		for i in self.match:
			l=self.match2clust(i)
			print i, l
			if len(l)>1 :
				sp.append((i,l))
		return sp

	def splitted_clust_dist(self):
		h={}
		for i in self.match:
			l=self.match2clust(i)
			print i, l
			nb_clust=len(l)
			if h.has_key(nb_clust):
				h[nb_clust]=h[nb_clust]+1
			else:
				h[nb_clust]=1
		return h

	def write_dist_splitted(self,filename):
		fileout=open(filename+".dist_split_group.dat",'w')
		h=self.splitted_group_dist()
		l=h.keys()
		l.sort()
		for i in l:
			fileout.write("%d\t%d\n"%(i,h[i]))
		fileout.close()
		
		fileout=open(filename+".dist_split_clust.dat",'w')
		h=self.splitted_clust_dist()
		l=h.keys()
		l.sort()
		for i in l:
			fileout.write("%d\t%d\n"%(i,h[i]))
		fileout.close()

	def write_group_info(self,filename,infoTE,grouplist=[]):
		l=self.rev_sort_group_sizes()
		fileout=open(filename,'w')
		for i in l:
			if i in grouplist or grouplist==[]:
				if self.group2match.has_key(i[2]):
					fileout.write("group:"+str(i[2])+\
						      " clust:"+str(self.group2clust[i[2]])+\
						      " size:"+str(i[0])+\
						      " length:"+str(i[1])+\
						      " TE:"+str(self.group2match[i[2]])
						      +"\n")
		fileout.close()

	def write_match_info(self,filename,infoTE):
		file=open(filename,"w")
		
		m2g=self.match2nbGroup()
		file.write("TE found in group: %d\n"%len(m2g))
		file.write("-----------------------\n")
		for i in m2g:
			file.write("%s in %d groups\n"%(i[0],i[1]))
			
		m2c=self.splitted_clust()
		file.write("\nTE found in several clusters: %d\n"%len(m2c))
		file.write("-----------------------------------\n")
		for i in m2c:
			file.write("%s in %s \n"%(i[0],i[1]))

		o2o=self.one2one()
		k=o2o.keys()
		file.write("\nOne TE found in one group: %d\n"%len(k))
		file.write("----------------------------------\n")
		k.sort()
		for i in k:
			file.write("%s in groups:%s\n"%(i,str(o2o[i])))

		na=self.neverAloneMatches()
		file.write("\nTE never alone in group: %d\n"%len(na))
		file.write("-------------------------------\n")
		for i in na:
			file.write("%-15s\n"%i)

		found,notfound=self.TE_found(infoTE)
		file.write("\nTE not found: %d\n"%len(notfound))
		file.write("---------------------\n")
		for i in notfound:
			file.write("%-15s\n"%i)
		file.close()






	

	







