import os
import unittest
import time
from FileUtils import FileUtils

class Test_FileUtils( unittest.TestCase ):
    
    def setUp( self ):
        self._uniqId = "%s_%s" % ( time.strftime("%Y%m%d%H%M%S"), os.getpid() )
        self.fileUtils = FileUtils()
        
        
    def tearDown( self ):
        if os.path.exists("dummyFolder"):
            os.system("chmod 777 dummyFolder")
            os.system("rm -r dummyFolder/")
        if os.path.exists("dummy.txt"):
            os.remove("dummy.txt")
        if os.path.exists("dummy1.txt"):
            os.remove("dummy1.txt")
        if os.path.exists("dummy2.txt"):
            os.remove("dummy2.txt")
        if os.path.exists("dummy3.txt"):
            os.remove("dummy3.txt")
            
            
    def testCountLinesInASingleFile( self ):
        # atomic
       f = open("dummy.txt", "w")
       f.write("line1\n")
       f.write("line2\n")
       f.write("line3")
       f.close()
       #file = FileChecker()
       
       self.assertEqual(3,self.fileUtils.countLinesInAFile("dummy.txt"))

       # empty file
       os.system("rm dummy.txt")   
       f = open("dummy.txt", "w")
       f.close()
   
       self.assertEqual(0,self.fileUtils.countLinesInAFile("dummy.txt"))
       os.system("rm dummy.txt") 
       
    def testCountLinesInManyFiles(self):
            # atomic
            f = open("dummy1.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
           
            f = open("dummy2.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
           
            f = open("dummy3.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
           
            filesList = ["dummy1.txt","dummy2.txt","dummy3.txt"]
        
           
            self.assertEqual(9,self.fileUtils.countLinesInFiles(filesList))
            
            
            # 2/3 empty files
            f = open("dummy1.txt", "w")
            f.close()
            f = open("dummy2.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
            f = open("dummy3.txt", "w")
            f.close()
           
            filesList = ["dummy1.txt","dummy2.txt","dummy3.txt"]
            fileChecker = FileUtils()
            self.assertEqual(3,self.fileUtils.countLinesInFiles(filesList))
            
            # 3/3 empty file
            f = open("dummy1.txt", "w")
            f.close()
            f = open("dummy2.txt", "w")
            f.close()
            f = open("dummy3.txt", "w")
            f.close()
           
            filesList = ["dummy1.txt","dummy2.txt","dummy3.txt"]
            self.assertEqual(0,self.fileUtils.countLinesInFiles(filesList))
           
            # empty file list
            filesList = []
            self.assertEqual(0,self.fileUtils.countLinesInFiles(filesList))
            # remove
            os.system("rm dummy1.txt")
            os.system("rm dummy2.txt")
            os.system("rm dummy3.txt")
            
    def testConcatFiles(self):
            
            f = open("dummy1.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
           
            f = open("dummy2.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
           
            f = open("dummy3.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
           
            filesList = ["dummy1.txt","dummy2.txt","dummy3.txt"]
            self.fileUtils.concatFiles(filesList, "concatFiles.txt")
            #test if file exists
            self.assertTrue(os.path.exists("concatFiles.txt"))   
            #test if file contains the good number of lines
            self.assertEqual(9,self.fileUtils.countLinesInAFile("concatFiles.txt"))
            # remove
            os.system("rm dummy1.txt")
            os.system("rm dummy2.txt")
            os.system("rm dummy3.txt")
            
            # test empty files
            f = open("dummy1.txt", "w")
            f.close()
            f = open("dummy2.txt", "w")
            f.close()
            f = open("dummy3.txt", "w")
            f.close() 
            filesList = ["dummy1.txt","dummy2.txt","dummy3.txt"]
            self.fileUtils.concatFiles(filesList, "concatFiles.txt")
            self.assertEqual(0,self.fileUtils.countLinesInAFile("concatFiles.txt"))
             # remove
            os.system("rm dummy1.txt")
            os.system("rm dummy2.txt")
            os.system("rm dummy3.txt")
           
           # test one empty file
            f = open("dummy1.txt", "w")
            f.close()
           
            f = open("dummy2.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
           
            f = open("dummy3.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
            
            filesList = ["dummy1.txt","dummy2.txt","dummy3.txt"]
            self.fileUtils.concatFiles(filesList, "concatFiles.txt")
            self.assertEqual(6,self.fileUtils.countLinesInAFile("concatFiles.txt"))
            os.system("rm concatFiles.txt")
            
    def testIsFileEmpty(self):  
            # test one empty file
            f = open("dummy1.txt", "w")
            f.close()
            self.assertTrue(self.fileUtils.isFileEmpty("dummy1.txt"))
            os.system("rm dummy1.txt")
            # test one no empty file
            f = open("dummy1.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
            self.assertFalse(self.fileUtils.isFileEmpty("dummy1.txt"))
            os.system("rm dummy1.txt")
            #test only one line
            f = open("dummy1.txt", "w")
            f.write("line1")
            f.close()
            self.assertFalse(self.fileUtils.isFileEmpty("dummy1.txt"))
            os.system("rm dummy1.txt")
            
    def testIsFileExists(self):  
            # file created
            f = open("dummy1.txt", "w")
            f.close()
            self.assertTrue(self.fileUtils.isRessourceExists("dummy1.txt"))
            os.system("rm dummy1.txt")
            # test deleted
            self.assertFalse(self.fileUtils.isRessourceExists("dummy1.txt"))
            
    def testIsDirExists(self):  
            # file created
            os.mkdir("dummyFolder")
            self.assertTrue(self.fileUtils.isRessourceExists("dummyFolder"))
            os.system("rm -r dummyFolder")
            # test deleted
            self.assertFalse(self.fileUtils.isRessourceExists("dummyFolder"))
    
    def testListDir(self):
            os.mkdir("dummyFolder")
            os.system("touch dummyFolder/dummySubject1.fa")
            os.system("touch dummyFolder/dummySubject2.fa")
            fileList = self.fileUtils.listDir("dummyFolder")
            self.assertEquals(2,len(fileList))
            os.system("rm -r dummyFolder")
    
    def testListDir_permission_denied_on_n_dir(self):
            os.mkdir("dummyFolder")
            os.system("touch dummyFolder/dummySubject1.fa")
            os.system("touch dummyFolder/dummySubject2.fa")
            os.system("chmod 000 dummyFolder")
            isExceptionRaised = False
            try:
                fileList = self.fileUtils.listDir("dummyFolder")
            except OSError, e:
                isExceptionRaised = True
                
            self.assertTrue(isExceptionRaised)
            self.assertEquals(e.strerror, "Permission denied")
            self.assertEquals(e.errno, 13)
            os.system("chmod 777 dummyFolder")
            os.system("rm -r dummyFolder/")
            
            
    def testListFilesInDir(self):
            os.mkdir("dummyFolder")
            os.system("touch dummyFolder/dummySubject1.fa")
            os.system("touch dummyFolder/dummySubject2.fa")
            os.system("mkdir dummyFolder/dummyFolder2")
            fileList = self.fileUtils.listFilesInDir("dummyFolder")
            self.assertEquals(2,len(fileList))
            fileList = self.fileUtils.listFilesInDir("dummyFolder", "dummySubject1.*")
            self.assertEquals(1,len(fileList))
            os.system("rm -r dummyFolder")       
    
    def testAre2FilesIdentical(self):
            #atomic case
            f = open("dummy1.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
            f = open("dummy2.txt", "w")
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3")
            f.close()
            self.assertTrue(self.fileUtils.are2FilesIdentical("dummy1.txt", "dummy2.txt"))
            #no identical file
            f = open("dummy2.txt", "w")
            f.write("line1\n")
            f.write("line1\n")
            f.write("line1")
            f.close()       
            self.assertFalse(self.fileUtils.are2FilesIdentical("dummy1.txt", "dummy2.txt"))
            #empty file
            f = open("dummy2.txt", "w")            
            f.close() 
            self.assertFalse(self.fileUtils.are2FilesIdentical("dummy1.txt", "dummy2.txt"))
            
    def test_are2FilesIdentical_with_inexistant_file(self):
        f = open("dummy.txt", "w")
        f.write("line1\n")
        f.close()
        self.assertFalse(self.fileUtils.are2FilesIdentical("dummy.txt", "dummy1.txt"))
            
    def testRemoveSuffix (self):
            if not (self.fileUtils.isRessourceExists("dummyFolder")):
                os.mkdir("dummyFolder")
            os.system("touch dummyFolder/dummySubject1.fa")
            os.system("touch dummyFolder/dummySubject1.fa.Nstretch.map")
            os.system("touch dummyFolder/dummySubject1.fa_cut")
            os.system("touch dummyFolder/dummySubject1.fa_cut.nhr")
            os.system("touch dummyFolder/dummySubject1.fa_cut.nin")
            os.system("touch dummyFolder/dummySubject1.fa_cut.nsq")
            suffixList = [".Nstretch.map", "_cut", "_cut.nhr", "_cut.nin", "_cut.nsq"]
            self.fileUtils.removeSuffix("dummyFolder/dummySubject1.fa", suffixList)
            self.assertFalse(self.fileUtils.isRessourceExists("dummyFolder/dummySubject1.fa.Nstretch.map"))
            self.assertFalse(self.fileUtils.isRessourceExists("dummyFolder/dummySubject1.fa_cut"))
            self.assertFalse(self.fileUtils.isRessourceExists("dummyFolder/dummySubject1.fa_cut.nhr"))
            self.assertFalse(self.fileUtils.isRessourceExists("dummyFolder/dummySubject1.fa_cut.nin"))
            self.assertFalse(self.fileUtils.isRessourceExists("dummyFolder/dummySubject1.fa_cut.nsq"))
            os.system("rm -r dummyFolder")
            
    def testRemoveSuffixWithNoFilePath (self):
            os.system("touch dummySubject1.fa")
            os.system("touch dummySubject1.fa.Nstretch.map")
            os.system("touch dummySubject1.fa_cut")
            os.system("touch dummySubject1.fa_cut.nhr")
            os.system("touch dummySubject1.fa_cut.nin")
            os.system("touch dummySubject1.fa_cut.nsq")    
            suffixList = [".Nstretch.map", "_cut", "_cut.nhr", "_cut.nin", "_cut.nsq"]
            self.fileUtils.removeSuffix("dummySubject1.fa", suffixList)
            self.assertFalse(self.fileUtils.isRessourceExists("dummySubject1.fa.Nstretch.map"))
            self.assertFalse(self.fileUtils.isRessourceExists("dummySubject1.fa_cut"))
            self.assertFalse(self.fileUtils.isRessourceExists("dummySubject1.fa_cut.nhr"))
            self.assertFalse(self.fileUtils.isRessourceExists("dummySubject1.fa_cut.nin"))
            self.assertFalse(self.fileUtils.isRessourceExists("dummySubject1.fa_cut.nsq"))
            os.remove("dummySubject1.fa")
    
    def testRemoveEmptyLine (self):
        
        inFileName="dummyWithEmptyLine.dum"
        
        obsEmptyLineFileName="dummyWithoutEmptyLine.dum"
        expFileName="dummyInFile.dum"
        
        self._writeFileWithEmptyLine(inFileName)
        self._writeFile(expFileName)
        
        self.fileUtils.removeEmptyLine (inFileName,obsEmptyLineFileName)
        
        self.assertTrue(self.fileUtils.are2FilesIdentical(expFileName, obsEmptyLineFileName))
        
        self.fileUtils.removeFile (obsEmptyLineFileName)
        self.fileUtils.removeFile (expFileName)
        self.fileUtils.removeFile (inFileName)
 
    def testRemoveEmptyLine_without_outfileName (self):
        inFileName="dummyWithEmptyLine.dum"
        expFileName="dummyInFile.dum"
        obsFileName = inFileName
        
        self._writeFileWithEmptyLine(inFileName)
        self._writeFile(expFileName)
        
        self.fileUtils.removeEmptyLine (inFileName,obsFileName)
        self.assertTrue(self.fileUtils.are2FilesIdentical(expFileName, obsFileName))
        
        self.fileUtils.removeFile (expFileName)
        self.fileUtils.removeFile (inFileName)
        
    def testRemoveRepeatedBlanks (self):
        inFileName="dummyWithRepeatedBlanks.dum"
        obsFileName="dummyWithoutRepeatedBlanks.dum"
        expFileName="dummyExpWithoutRepeatedBlanks.dum"
        
        self._writeFileWithRepeatedBlanks(inFileName)
        self._writeFileWithoutRepeatedBlanks (expFileName)
        
        self.fileUtils.removeRepeatedBlanks (inFileName,obsFileName)
        self.assertTrue(self.fileUtils.are2FilesIdentical(expFileName, obsFileName))
        
        self.fileUtils.removeFile (expFileName)
        self.fileUtils.removeFile (obsFileName)                   
        self.fileUtils.removeFile (inFileName)
     
    def testRemoveRepeatedBlanks_without_outfileName (self):
        inFileName="dummyWithRepeatedBlanks.dum"
        expFileName="dummyExpWithoutRepeatedBlanks.dum"
        obsFileName = inFileName
        
        self._writeFileWithRepeatedBlanks (inFileName)
        self._writeFileWithoutRepeatedBlanks (expFileName)

        self.fileUtils.removeRepeatedBlanks (inFileName)
        self.assertTrue (self.fileUtils.are2FilesIdentical(expFileName, obsFileName))
        
        self.fileUtils.removeFile (expFileName)
        self.fileUtils.removeFile (inFileName)  
            
    def testListDirWithPattern(self):
            if not (self.fileUtils.isRessourceExists("dummyFolder")):
                os.mkdir("dummyFolder")
            os.system("touch dummyFolder/batch_1.fa.align")
            os.system("touch dummyFolder/batch_2.fa.align")
            os.system("touch dummyFolder/batch_3.fa.align")
            os.system("touch dummyFolder/batch_4.fa.align")
            os.system("touch dummyFolder/batch_5.fa.align")
            os.system("touch dummyFolder/batch_6.fa.align")
            os.system("touch dummyFolder/batch_1.fa")
            os.system("touch dummyFolder/batch_2.fa")
            os.system("touch dummyFolder/batch_3.fa")
            os.system("touch dummyFolder/batch_4.fa")
            os.system("touch dummyFolder/batch_5.fa")
            os.system("touch dummyFolder/batch_6.fa")
            
            fileList = self.fileUtils.listDir("dummyFolder", "batch_\d.+\.align")
            self.assertEquals(len(fileList), 6)
            os.system("rm -rf dummyFolder")

    def test_catFilesUsingPattern_static( self ):
        file1 = "dummyFile1_%s" % ( self._uniqId )
        file1Handler = open( file1, "w" )
        file1Handler.write( "dhdgdgetcdhzbvdey\n" )
        file1Handler.close()
        
        file2 = "dummyFile2_%s" % ( self._uniqId )
        file2Handler = open( file2, "w" )
        file2Handler.write( "bjkfrevngtrhkvnjvf\n" )
        file2Handler.close()
        
        expFile1 = "dummyExpFile_%s" % ( self._uniqId ) + "_1"
        expFileHandler = open( expFile1, "w" )
        expFileHandler.write( "dhdgdgetcdhzbvdey\n" )
        expFileHandler.write( "bjkfrevngtrhkvnjvf\n" )
        expFileHandler.close()
        
        expFile2 = "dummyExpFile_%s" % ( self._uniqId ) + "_2"
        expFileHandler = open( expFile2, "w" )
        expFileHandler.write( "bjkfrevngtrhkvnjvf\n" )
        expFileHandler.write( "dhdgdgetcdhzbvdey\n" )
        expFileHandler.close()
        
        obsFile = "dummyObsFile_%s" % ( self._uniqId )
        
        FileUtils.catFilesUsingPattern( "dummyFile*_%s" % ( self._uniqId ), obsFile )
        
        self.assertTrue( FileUtils.are2FilesIdentical_static( expFile1, obsFile )\
                          or FileUtils.are2FilesIdentical_static(expFile2, obsFile) )
        
        for f in [ file1, file2, expFile1, expFile2, obsFile ]:
            if os.path.exists( f ):
                os.remove( f )
                
#--------------------------------------------------------------
     
    def _writeFile(self, fileName):
        inFile = open(fileName, 'w')
        inFile.write(">Sequence_de_reference\n")
        inFile.write("ATTTTGCAGTCTTATTCGAG-----GCCATTGCT\n")
        inFile.write(">Lignee1_mismatch\n")
        inFile.write("ATTTTGCAGACTTATTCGAG-----GCCATTGCT\n")
        inFile.write(">Lignee2_insertion\n")
        inFile.write("ATTTTGCAGTCTTATTCGAGATTACGCCATTGCT\n")
        inFile.write(">Lignee3_deletion\n")
        inFile.write("A---TGCAGTCTTATTCGAG-----GCCATTGCT\n")
        inFile.close()      
     
    def _writeFileWithEmptyLine(self, fileName):
        fileWithEmptyLine = open(fileName, 'w')
        fileWithEmptyLine.write(">Sequence_de_reference\n")
        fileWithEmptyLine.write("ATTTTGCAGTCTTATTCGAG-----GCCATTGCT\n")
        fileWithEmptyLine.write("\n\n")
        fileWithEmptyLine.write(">Lignee1_mismatch\n")
        fileWithEmptyLine.write("ATTTTGCAGACTTATTCGAG-----GCCATTGCT\n")
        fileWithEmptyLine.write("\n\n")
        fileWithEmptyLine.write(">Lignee2_insertion\n")
        fileWithEmptyLine.write("ATTTTGCAGTCTTATTCGAGATTACGCCATTGCT\n")
        fileWithEmptyLine.write("\n")
        fileWithEmptyLine.write(">Lignee3_deletion\n")
        fileWithEmptyLine.write("A---TGCAGTCTTATTCGAG-----GCCATTGCT\n")
        fileWithEmptyLine.close() 
           
    def _writeFileWithRepeatedBlanks(self, fileName):
        fileWithRepeatedBlanks = open(fileName, 'w')
        fileWithRepeatedBlanks.write(">Sequ  ence_de     _reference\n")
        fileWithRepeatedBlanks.write("ATTTT  GCAGTCTT TTCGAG-  ----GCCATT  GCT\n")
        fileWithRepeatedBlanks.close() 
    
    def _writeFileWithoutRepeatedBlanks (self, fileName):
        fileWithoutRepeatedBlanks = open(fileName, 'w')
        fileWithoutRepeatedBlanks.write(">Sequ ence_de _reference\n")
        fileWithoutRepeatedBlanks.write("ATTTT GCAGTCTT TTCGAG- ----GCCATT GCT\n")
        fileWithoutRepeatedBlanks.close() 
         
test_suite = unittest.TestSuite()
test_suite.addTest( unittest.makeSuite( Test_FileUtils ) )
if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run( test_suite )
